@@ -1,58 +1,17 @@
 /*
- * The Apache Software License, Version 1.1
- *
- *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.apache.org.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2002,2004,2005 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package dom;
@@ -63,7 +22,6 @@ import org.w3c.dom.Document;
 import org.w3c.dom.NamedNodeMap;
 import org.w3c.dom.Node;
 import org.w3c.dom.Text;
-
 import org.xml.sax.SAXException;
 import org.xml.sax.SAXParseException;
 
@@ -87,7 +45,7 @@ import org.xml.sax.SAXParseException;
  *
  * @author Andy Clark, IBM
  *
- * @version $Id: Counter.java,v 1.9 2003/11/29 18:07:34 mrglavas Exp $
+ * @version $Id: Counter.java 329525 2005-10-30 05:39:22Z mrglavas $
  */
 public class Counter {
 
@@ -109,8 +67,23 @@ public class Counter {
     /** Schema full checking feature id (http://apache.org/xml/features/validation/schema-full-checking). */
     protected static final String SCHEMA_FULL_CHECKING_FEATURE_ID = "http://apache.org/xml/features/validation/schema-full-checking";
     
+    /** Honour all schema locations feature id (http://apache.org/xml/features/honour-all-schemaLocations). */
+    protected static final String HONOUR_ALL_SCHEMA_LOCATIONS_ID = "http://apache.org/xml/features/honour-all-schemaLocations";
+    
+    /** Validate schema annotations feature id (http://apache.org/xml/features/validate-annotations). */
+    protected static final String VALIDATE_ANNOTATIONS_ID = "http://apache.org/xml/features/validate-annotations";
+    
     /** Dynamic validation feature id (http://apache.org/xml/features/validation/dynamic). */
     protected static final String DYNAMIC_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/dynamic";
+    
+    /** XInclude feature id (http://apache.org/xml/features/xinclude). */
+    protected static final String XINCLUDE_FEATURE_ID = "http://apache.org/xml/features/xinclude";
+    
+    /** XInclude fixup base URIs feature id (http://apache.org/xml/features/xinclude/fixup-base-uris). */
+    protected static final String XINCLUDE_FIXUP_BASE_URIS_FEATURE_ID = "http://apache.org/xml/features/xinclude/fixup-base-uris";
+    
+    /** XInclude fixup language feature id (http://apache.org/xml/features/xinclude/fixup-language). */
+    protected static final String XINCLUDE_FIXUP_LANGUAGE_FEATURE_ID = "http://apache.org/xml/features/xinclude/fixup-language";
 
     // default settings
 
@@ -131,9 +104,24 @@ public class Counter {
 
     /** Default Schema full checking support (false). */
     protected static final boolean DEFAULT_SCHEMA_FULL_CHECKING = false;
+    
+    /** Default honour all schema locations (false). */
+    protected static final boolean DEFAULT_HONOUR_ALL_SCHEMA_LOCATIONS = false;
+    
+    /** Default validate schema annotations (false). */
+    protected static final boolean DEFAULT_VALIDATE_ANNOTATIONS = false;
 
     /** Default dynamic validation support (false). */
     protected static final boolean DEFAULT_DYNAMIC_VALIDATION = false;
+    
+    /** Default XInclude processing support (false). */
+    protected static final boolean DEFAULT_XINCLUDE = false;
+    
+    /** Default XInclude fixup base URIs support (true). */
+    protected static final boolean DEFAULT_XINCLUDE_FIXUP_BASE_URIS = true;
+    
+    /** Default XInclude fixup language support (true). */
+    protected static final boolean DEFAULT_XINCLUDE_FIXUP_LANGUAGE = true;
 
     //
     // Data
@@ -281,7 +269,12 @@ public class Counter {
         boolean validation = DEFAULT_VALIDATION;
         boolean schemaValidation = DEFAULT_SCHEMA_VALIDATION;
         boolean schemaFullChecking = DEFAULT_SCHEMA_FULL_CHECKING;
+        boolean honourAllSchemaLocations = DEFAULT_HONOUR_ALL_SCHEMA_LOCATIONS;
+        boolean validateAnnotations = DEFAULT_VALIDATE_ANNOTATIONS;
         boolean dynamicValidation = DEFAULT_DYNAMIC_VALIDATION;
+        boolean xincludeProcessing = DEFAULT_XINCLUDE;
+        boolean xincludeFixupBaseURIs = DEFAULT_XINCLUDE_FIXUP_BASE_URIS;
+        boolean xincludeFixupLanguage = DEFAULT_XINCLUDE_FIXUP_LANGUAGE;
 
         // process arguments
         for (int i = 0; i < argv.length; i++) {
@@ -340,10 +333,30 @@ public class Counter {
                     schemaFullChecking = option.equals("f");
                     continue;
                 }
+                if (option.equalsIgnoreCase("hs")) {
+                    honourAllSchemaLocations = option.equals("hs");
+                    continue;
+                }
+                if (option.equalsIgnoreCase("va")) {
+                    validateAnnotations = option.equals("va");
+                    continue;
+                }
                 if (option.equalsIgnoreCase("dv")) {
                     dynamicValidation = option.equals("dv");
                     continue;
                 }
+                if (option.equalsIgnoreCase("xi")) {
+                    xincludeProcessing = option.equals("xi");
+                    continue;
+                }
+                if (option.equalsIgnoreCase("xb")) {
+                    xincludeFixupBaseURIs = option.equals("xb");
+                    continue;
+                }
+                if (option.equalsIgnoreCase("xl")) {
+                    xincludeFixupLanguage = option.equals("xl");
+                    continue;
+                }
                 if (option.equals("h")) {
                     printUsage();
                     continue;
@@ -388,12 +401,42 @@ public class Counter {
             catch (SAXException e) {
                 System.err.println("warning: Parser does not support feature ("+SCHEMA_FULL_CHECKING_FEATURE_ID+")");
             }
+            try {
+                parser.setFeature(HONOUR_ALL_SCHEMA_LOCATIONS_ID, honourAllSchemaLocations);
+            }
+            catch (SAXException e) {
+                System.err.println("warning: Parser does not support feature ("+HONOUR_ALL_SCHEMA_LOCATIONS_ID+")");
+            }
+            try {
+                parser.setFeature(VALIDATE_ANNOTATIONS_ID, validateAnnotations);
+            }
+            catch (SAXException e) {
+                System.err.println("warning: Parser does not support feature ("+VALIDATE_ANNOTATIONS_ID+")");
+            }
             try {
                 parser.setFeature(DYNAMIC_VALIDATION_FEATURE_ID, dynamicValidation);
             }
             catch (SAXException e) {
                 System.err.println("warning: Parser does not support feature ("+DYNAMIC_VALIDATION_FEATURE_ID+")");
             }
+            try {
+                parser.setFeature(XINCLUDE_FEATURE_ID, xincludeProcessing);
+            }
+            catch (SAXException e) {
+                System.err.println("warning: Parser does not support feature ("+XINCLUDE_FEATURE_ID+")");
+            }
+            try {
+                parser.setFeature(XINCLUDE_FIXUP_BASE_URIS_FEATURE_ID, xincludeFixupBaseURIs);
+            }
+            catch (SAXException e) {
+                System.err.println("warning: Parser does not support feature ("+XINCLUDE_FIXUP_BASE_URIS_FEATURE_ID+")");
+            }
+            try {
+                parser.setFeature(XINCLUDE_FIXUP_LANGUAGE_FEATURE_ID, xincludeFixupLanguage);
+            }
+            catch (SAXException e) {
+                System.err.println("warning: Parser does not support feature ("+XINCLUDE_FIXUP_LANGUAGE_FEATURE_ID+")");
+            }
 
             // parse file
             try {
@@ -457,8 +500,18 @@ public class Counter {
         System.err.println("              NOTE: Not supported by all parsers.");
         System.err.println("  -f  | -F    Turn on/off Schema full checking.");
         System.err.println("              NOTE: Requires use of -s and not supported by all parsers.");
+        System.err.println("  -hs | -HS   Turn on/off honouring of all schema locations.");
+        System.err.println("              NOTE: Requires use of -s and not supported by all parsers.");
+        System.err.println("  -va | -VA   Turn on/off validation of schema annotations.");
+        System.err.println("              NOTE: Requires use of -s and not supported by all parsers.");
         System.err.println("  -dv | -DV   Turn on/off dynamic validation.");
         System.err.println("              NOTE: Not supported by all parsers.");
+        System.err.println("  -xi | -XI   Turn on/off XInclude processing.");
+        System.err.println("              NOTE: Not supported by all parsers.");
+        System.err.println("  -xb | -XB   Turn on/off base URI fixup during XInclude processing.");
+        System.err.println("              NOTE: Requires use of -xi and not supported by all parsers.");
+        System.err.println("  -xl | -XL   Turn on/off language fixup during XInclude processing.");
+        System.err.println("              NOTE: Requires use of -xi and not supported by all parsers.");
         System.err.println("  -h          This help screen.");
         System.err.println();
 
@@ -471,10 +524,20 @@ public class Counter {
         System.err.println(DEFAULT_VALIDATION ? "on" : "off");
         System.err.print("  Schema:     ");
         System.err.println(DEFAULT_SCHEMA_VALIDATION ? "on" : "off");
-        System.err.print("  Schema full checking:     ");
+        System.err.print("  Schema full checking:            ");
         System.err.println(DEFAULT_SCHEMA_FULL_CHECKING ? "on" : "off");
+        System.err.print("  Honour all schema locations:     ");
+        System.err.println(DEFAULT_HONOUR_ALL_SCHEMA_LOCATIONS ? "on" : "off");
+        System.err.print("  Validate annotations:            ");
+        System.err.println(DEFAULT_VALIDATE_ANNOTATIONS ? "on" : "off");
         System.err.print("  Dynamic:    ");
         System.err.println(DEFAULT_DYNAMIC_VALIDATION ? "on" : "off");
+        System.err.print("  XInclude:   ");
+        System.err.println(DEFAULT_XINCLUDE ? "on" : "off");
+        System.err.print("  XInclude base URI fixup:  ");
+        System.err.println(DEFAULT_XINCLUDE_FIXUP_BASE_URIS ? "on" : "off");
+        System.err.print("  XInclude language fixup:  ");
+        System.err.println(DEFAULT_XINCLUDE_FIXUP_LANGUAGE ? "on" : "off");
 
     } // printUsage()
 
