@@ -1,58 +1,17 @@
 /*
- * The Apache Software License, Version 1.1
- *
- *
- * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.apache.org.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2006 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package sax;
@@ -87,7 +46,7 @@ import org.xml.sax.helpers.XMLReaderFactory;
  *
  * @author Andy Clark, IBM
  *
- * @version $Id: Writer.java,v 1.10 2003/11/29 16:19:24 mrglavas Exp $
+ * @version $Id: Writer.java 429930 2006-08-09 02:33:11Z mrglavas $
  */
 public class Writer
     extends DefaultHandler
@@ -113,12 +72,30 @@ public class Writer
 
     /** Schema full checking feature id (http://apache.org/xml/features/validation/schema-full-checking). */
     protected static final String SCHEMA_FULL_CHECKING_FEATURE_ID = "http://apache.org/xml/features/validation/schema-full-checking";
+    
+    /** Honour all schema locations feature id (http://apache.org/xml/features/honour-all-schemaLocations). */
+    protected static final String HONOUR_ALL_SCHEMA_LOCATIONS_ID = "http://apache.org/xml/features/honour-all-schemaLocations";
+    
+    /** Validate schema annotations feature id (http://apache.org/xml/features/validate-annotations) */
+    protected static final String VALIDATE_ANNOTATIONS_ID = "http://apache.org/xml/features/validate-annotations";
+    
+    /** Generate synthetic schema annotations feature id (http://apache.org/xml/features/generate-synthetic-annotations). */
+    protected static final String GENERATE_SYNTHETIC_ANNOTATIONS_ID = "http://apache.org/xml/features/generate-synthetic-annotations";
 
     /** Dynamic validation feature id (http://apache.org/xml/features/validation/dynamic). */
     protected static final String DYNAMIC_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/dynamic";
     
     /** Load external DTD feature id (http://apache.org/xml/features/nonvalidating/load-external-dtd). */
     protected static final String LOAD_EXTERNAL_DTD_FEATURE_ID = "http://apache.org/xml/features/nonvalidating/load-external-dtd";
+    
+    /** XInclude feature id (http://apache.org/xml/features/xinclude). */
+    protected static final String XINCLUDE_FEATURE_ID = "http://apache.org/xml/features/xinclude";
+    
+    /** XInclude fixup base URIs feature id (http://apache.org/xml/features/xinclude/fixup-base-uris). */
+    protected static final String XINCLUDE_FIXUP_BASE_URIS_FEATURE_ID = "http://apache.org/xml/features/xinclude/fixup-base-uris";
+    
+    /** XInclude fixup language feature id (http://apache.org/xml/features/xinclude/fixup-language). */
+    protected static final String XINCLUDE_FIXUP_LANGUAGE_FEATURE_ID = "http://apache.org/xml/features/xinclude/fixup-language";
 
     // property ids
 
@@ -148,8 +125,26 @@ public class Writer
     /** Default Schema full checking support (false). */
     protected static final boolean DEFAULT_SCHEMA_FULL_CHECKING = false;
     
+    /** Default honour all schema locations (false). */
+    protected static final boolean DEFAULT_HONOUR_ALL_SCHEMA_LOCATIONS = false;
+    
+    /** Default validate schema annotations (false). */
+    protected static final boolean DEFAULT_VALIDATE_ANNOTATIONS = false;
+    
+    /** Default generate synthetic schema annotations (false). */
+    protected static final boolean DEFAULT_GENERATE_SYNTHETIC_ANNOTATIONS = false;
+    
     /** Default dynamic validation support (false). */
     protected static final boolean DEFAULT_DYNAMIC_VALIDATION = false;
+    
+    /** Default XInclude processing support (false). */
+    protected static final boolean DEFAULT_XINCLUDE = false;
+    
+    /** Default XInclude fixup base URIs support (true). */
+    protected static final boolean DEFAULT_XINCLUDE_FIXUP_BASE_URIS = true;
+    
+    /** Default XInclude fixup language support (true). */
+    protected static final boolean DEFAULT_XINCLUDE_FIXUP_LANGUAGE = true;
 
     /** Default canonical output (false). */
     protected static final boolean DEFAULT_CANONICAL = false;
@@ -488,7 +483,7 @@ public class Writer
                 // else, default print char
             }
             default: {
-           	// In XML 1.1, control chars in the ranges [#x1-#x1F, #x7F-#x9F] must be escaped.
+                // In XML 1.1, control chars in the ranges [#x1-#x1F, #x7F-#x9F] must be escaped.
             	//
             	// Escape space characters that would be normalized to #x20 in attribute values
             	// when the document is reparsed.
@@ -545,7 +540,7 @@ public class Writer
             getXMLVersion = fLocator.getClass().getMethod("getXMLVersion", new Class[]{});
             // If Locator implements Locator2, this method will exist.
             if (getXMLVersion != null) {
-                version = (String) getXMLVersion.invoke(fLocator, null);
+                version = (String) getXMLVersion.invoke(fLocator, (Object[]) null);
             }
         } 
         catch (Exception e) { 
@@ -577,7 +572,13 @@ public class Writer
         boolean externalDTD = DEFAULT_LOAD_EXTERNAL_DTD;
         boolean schemaValidation = DEFAULT_SCHEMA_VALIDATION;
         boolean schemaFullChecking = DEFAULT_SCHEMA_FULL_CHECKING;
+        boolean honourAllSchemaLocations = DEFAULT_HONOUR_ALL_SCHEMA_LOCATIONS;
+        boolean validateAnnotations = DEFAULT_VALIDATE_ANNOTATIONS;
+        boolean generateSyntheticAnnotations = DEFAULT_GENERATE_SYNTHETIC_ANNOTATIONS;
         boolean dynamicValidation = DEFAULT_DYNAMIC_VALIDATION;
+        boolean xincludeProcessing = DEFAULT_XINCLUDE;
+        boolean xincludeFixupBaseURIs = DEFAULT_XINCLUDE_FIXUP_BASE_URIS;
+        boolean xincludeFixupLanguage = DEFAULT_XINCLUDE_FIXUP_LANGUAGE;
         boolean canonical = DEFAULT_CANONICAL;
 
         // process arguments
@@ -634,10 +635,34 @@ public class Writer
                     schemaFullChecking = option.equals("f");
                     continue;
                 }
+                if (option.equalsIgnoreCase("hs")) {
+                    honourAllSchemaLocations = option.equals("hs");
+                    continue;
+                }
+                if (option.equalsIgnoreCase("va")) {
+                    validateAnnotations = option.equals("va");
+                    continue;
+                }
+                if (option.equalsIgnoreCase("ga")) {
+                    generateSyntheticAnnotations = option.equals("ga");
+                    continue;
+                }
                 if (option.equalsIgnoreCase("dv")) {
                     dynamicValidation = option.equals("dv");
                     continue;
                 }
+                if (option.equalsIgnoreCase("xi")) {
+                    xincludeProcessing = option.equals("xi");
+                    continue;
+                }
+                if (option.equalsIgnoreCase("xb")) {
+                    xincludeFixupBaseURIs = option.equals("xb");
+                    continue;
+                }
+                if (option.equalsIgnoreCase("xl")) {
+                    xincludeFixupLanguage = option.equals("xl");
+                    continue;
+                }
                 if (option.equalsIgnoreCase("c")) {
                     canonical = option.equals("c");
                     continue;
@@ -708,6 +733,33 @@ public class Writer
             catch (SAXNotSupportedException e) {
                 System.err.println("warning: Parser does not support feature ("+SCHEMA_FULL_CHECKING_FEATURE_ID+")");
             }
+            try {
+                parser.setFeature(HONOUR_ALL_SCHEMA_LOCATIONS_ID, honourAllSchemaLocations);
+            }
+            catch (SAXNotRecognizedException e) {
+                System.err.println("warning: Parser does not recognize feature ("+HONOUR_ALL_SCHEMA_LOCATIONS_ID+")");
+            }
+            catch (SAXNotSupportedException e) {
+                System.err.println("warning: Parser does not support feature ("+HONOUR_ALL_SCHEMA_LOCATIONS_ID+")");
+            }
+            try {
+                parser.setFeature(VALIDATE_ANNOTATIONS_ID, validateAnnotations);
+            }
+            catch (SAXNotRecognizedException e) {
+                System.err.println("warning: Parser does not recognize feature ("+VALIDATE_ANNOTATIONS_ID+")");
+            }
+            catch (SAXNotSupportedException e) {
+                System.err.println("warning: Parser does not support feature ("+VALIDATE_ANNOTATIONS_ID+")");
+            }
+            try {
+                parser.setFeature(GENERATE_SYNTHETIC_ANNOTATIONS_ID, generateSyntheticAnnotations);
+            }
+            catch (SAXNotRecognizedException e) {
+                System.err.println("warning: Parser does not recognize feature ("+GENERATE_SYNTHETIC_ANNOTATIONS_ID+")");
+            }
+            catch (SAXNotSupportedException e) {
+                System.err.println("warning: Parser does not support feature ("+GENERATE_SYNTHETIC_ANNOTATIONS_ID+")");
+            }
             try {
                 parser.setFeature(DYNAMIC_VALIDATION_FEATURE_ID, dynamicValidation);
             }
@@ -717,6 +769,33 @@ public class Writer
             catch (SAXNotSupportedException e) {
                 System.err.println("warning: Parser does not support feature ("+DYNAMIC_VALIDATION_FEATURE_ID+")");
             }
+            try {
+                parser.setFeature(XINCLUDE_FEATURE_ID, xincludeProcessing);
+            }
+            catch (SAXNotRecognizedException e) {
+                System.err.println("warning: Parser does not recognize feature ("+XINCLUDE_FEATURE_ID+")");
+            }
+            catch (SAXNotSupportedException e) {
+                System.err.println("warning: Parser does not support feature ("+XINCLUDE_FEATURE_ID+")");
+            }
+            try {
+                parser.setFeature(XINCLUDE_FIXUP_BASE_URIS_FEATURE_ID, xincludeFixupBaseURIs);
+            }
+            catch (SAXNotRecognizedException e) {
+                System.err.println("warning: Parser does not recognize feature ("+XINCLUDE_FIXUP_BASE_URIS_FEATURE_ID+")");
+            }
+            catch (SAXNotSupportedException e) {
+                System.err.println("warning: Parser does not support feature ("+XINCLUDE_FIXUP_BASE_URIS_FEATURE_ID+")");
+            }
+            try {
+                parser.setFeature(XINCLUDE_FIXUP_LANGUAGE_FEATURE_ID, xincludeFixupLanguage);
+            }
+            catch (SAXNotRecognizedException e) {
+                System.err.println("warning: Parser does not recognize feature ("+XINCLUDE_FIXUP_LANGUAGE_FEATURE_ID+")");
+            }
+            catch (SAXNotSupportedException e) {
+                System.err.println("warning: Parser does not support feature ("+XINCLUDE_FIXUP_LANGUAGE_FEATURE_ID+")");
+            }
             
             // setup writer
             if (writer == null) {
@@ -774,18 +853,30 @@ public class Writer
 
         System.err.println("options:");
         System.err.println("  -p name     Select parser by name.");
-        System.err.println("  -n | -N     Turn on/off namespace processing.");
+        System.err.println("  -n  | -N    Turn on/off namespace processing.");
         System.err.println("  -np | -NP   Turn on/off namespace prefixes.");
         System.err.println("              NOTE: Requires use of -n.");
-        System.err.println("  -v | -V     Turn on/off validation.");
+        System.err.println("  -v  | -V    Turn on/off validation.");
         System.err.println("  -xd | -XD   Turn on/off loading of external DTDs.");
         System.err.println("              NOTE: Always on when -v in use and not supported by all parsers.");
-        System.err.println("  -s | -S     Turn on/off Schema validation support.");
+        System.err.println("  -s  | -S    Turn on/off Schema validation support.");
         System.err.println("              NOTE: Not supported by all parsers.");
         System.err.println("  -f  | -F    Turn on/off Schema full checking.");
         System.err.println("              NOTE: Requires use of -s and not supported by all parsers.");
+        System.err.println("  -hs | -HS   Turn on/off honouring of all schema locations.");
+        System.err.println("              NOTE: Requires use of -s and not supported by all parsers.");
+        System.err.println("  -va | -VA   Turn on/off validation of schema annotations.");
+        System.err.println("              NOTE: Requires use of -s and not supported by all parsers.");
+        System.err.println("  -ga | -GA   Turn on/off generation of synthetic schema annotations.");
+        System.err.println("              NOTE: Requires use of -s and not supported by all parsers.");
         System.err.println("  -dv | -DV   Turn on/off dynamic validation.");
         System.err.println("              NOTE: Not supported by all parsers.");
+        System.err.println("  -xi | -XI   Turn on/off XInclude processing.");
+        System.err.println("              NOTE: Not supported by all parsers.");
+        System.err.println("  -xb | -XB   Turn on/off base URI fixup during XInclude processing.");
+        System.err.println("              NOTE: Requires use of -xi and not supported by all parsers.");
+        System.err.println("  -xl | -XL   Turn on/off language fixup during XInclude processing.");
+        System.err.println("              NOTE: Requires use of -xi and not supported by all parsers.");
         System.err.println("  -c | -C     Turn on/off Canonical XML output.");
         System.err.println("              NOTE: This is not W3C canonical output.");
         System.err.println("  -h          This help screen.");
@@ -809,6 +900,18 @@ public class Writer
         System.err.println(DEFAULT_DYNAMIC_VALIDATION ? "on" : "off");
         System.err.print("  Canonical:  ");
         System.err.println(DEFAULT_CANONICAL ? "on" : "off");
+        System.err.print("  Honour all schema locations:       ");
+        System.err.println(DEFAULT_HONOUR_ALL_SCHEMA_LOCATIONS ? "on" : "off");
+        System.err.print("  Validate Annotations:              ");
+        System.err.println(DEFAULT_VALIDATE_ANNOTATIONS ? "on" : "off");
+        System.err.print("  Generate Synthetic Annotations:    ");
+        System.err.println(DEFAULT_GENERATE_SYNTHETIC_ANNOTATIONS ? "on" : "off");
+        System.err.print("  XInclude:   ");
+        System.err.println(DEFAULT_XINCLUDE ? "on" : "off");
+        System.err.print("  XInclude base URI fixup:  ");
+        System.err.println(DEFAULT_XINCLUDE_FIXUP_BASE_URIS ? "on" : "off");
+        System.err.print("  XInclude language fixup:  ");
+        System.err.println(DEFAULT_XINCLUDE_FIXUP_LANGUAGE ? "on" : "off");
 
     } // printUsage()
 
