@@ -1,68 +1,28 @@
 /*
- * The Apache Software License, Version 1.1
- *
- *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.apache.org.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2002,2004-2006 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package org.apache.xerces.dom;
 
-import java.io.Serializable;
+import java.lang.reflect.Constructor;
 import java.util.Enumeration;
 import java.util.Hashtable;
+import org.apache.xerces.util.URI;
 
-import org.apache.xerces.dom3.DOMConfiguration;
-import org.apache.xerces.dom3.UserDataHandler;
+import org.w3c.dom.DOMConfiguration;
+import org.w3c.dom.UserDataHandler;
 import org.apache.xerces.util.XMLChar;
 import org.apache.xerces.util.XML11Char;
 import org.apache.xerces.xni.NamespaceContext;
@@ -105,12 +65,14 @@ import org.w3c.dom.ls.LSSerializer;
  * <p>
  * <b>Note:</b> When any node in the document is serialized, the
  * entire document is serialized along with it.
+ * 
+ * @xerces.internal 
  *
  * @author Arnaud  Le Hors, IBM
  * @author Joe Kesselman, IBM
  * @author Andy Clark, IBM
  * @author Ralf Pfeiffer, IBM
- * @version $Id: CoreDocumentImpl.java,v 1.64 2004/02/17 07:14:48 neeraj Exp $
+ * @version $Id: CoreDocumentImpl.java 439176 2006-09-01 04:45:24Z mrglavas $
  * @since  PR-DOM-Level-1-19980818.
  */
 
@@ -161,17 +123,19 @@ extends ParentNode implements Document  {
     /**Experimental DOM Level 3 feature: documentURI */
     protected String fDocumentURI;
 
+	//Revisit :: change to a better data structure.
     /** Table for user data attached to this document nodes. */
     protected Hashtable userData;
 
+
     /** Identifiers. */
     protected Hashtable identifiers;
 
     // DOM Level 3: normalizeDocument
     transient DOMNormalizer domNormalizer = null;
-    transient DOMConfigurationImpl fConfiguration= null;
-
-    // support of XPath API
+    transient DOMConfigurationImpl fConfiguration = null;
+    
+    // support of XPath API   
     transient Object fXPathEvaluator = null;
 
     /** Table for quick check of child insertion. */
@@ -411,6 +375,9 @@ extends ParentNode implements Document  {
         // Only one such child permitted
         int type = newChild.getNodeType();
         if (errorChecking) {
+            if (needsSyncChildren()) {
+                synchronizeChildren();
+            }
             if((type == Node.ELEMENT_NODE && docElement != null) ||
             (type == Node.DOCUMENT_TYPE_NODE && docType != null)) {
                 String msg = DOMMessageFormatter.formatMessage(DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null);
@@ -475,6 +442,18 @@ extends ParentNode implements Document  {
         newChild instanceof DocumentTypeImpl) {
             ((DocumentTypeImpl) newChild).ownerDocument = this;
         }
+
+        if (errorChecking &&((docType != null &&
+            oldChild.getNodeType() != Node.DOCUMENT_TYPE_NODE && 
+            newChild.getNodeType() == Node.DOCUMENT_TYPE_NODE) 
+            || (docElement != null && 
+            oldChild.getNodeType() != Node.ELEMENT_NODE && 
+            newChild.getNodeType() == Node.ELEMENT_NODE))) {
+            	
+            throw new DOMException(
+                DOMException.HIERARCHY_REQUEST_ERR,
+                DOMMessageFormatter.formatMessage(DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
+        }
         super.replaceChild(newChild, oldChild);
 
         int type = oldChild.getNodeType();
@@ -508,19 +487,42 @@ extends ParentNode implements Document  {
      * @since DOM Level 3
      */
     public Object getFeature(String feature, String version) {
-        if ((feature.equalsIgnoreCase("XPath")
-        || feature.equalsIgnoreCase("+XPath"))&& version.equals("3.0")){
-            try{
-                Class xpathClass = ObjectFactory.findProviderClass(
-                "org.apache.xpath.domapi.XPathEvaluatorImpl",
-                ObjectFactory.findClassLoader(), true);
-                fXPathEvaluator = xpathClass.newInstance();
-                java.lang.reflect.Method setDocument = xpathClass.getMethod("setDoc", new Class[]{Document.class});
-                setDocument.invoke(fXPathEvaluator, new Object[]{this});
+        
+        boolean anyVersion = version == null || version.length() == 0;
+        
+        // if a plus sign "+" is prepended to any feature name, implementations
+        // are considered in which the specified feature may not be directly
+        // castable DOMImplementation.getFeature(feature, version). Without a
+        // plus, only features whose interfaces are directly castable are
+        // considered.
+        if ((feature.equalsIgnoreCase("+XPath"))
+            && (anyVersion || version.equals("3.0"))) {
+            
+            // If an XPathEvaluator was created previously 
+            // return it otherwise create a new one.
+            if (fXPathEvaluator != null) {
                 return fXPathEvaluator;
             }
-            catch (Exception e){
-                //e.printStackTrace();
+            
+            try {
+                Class xpathClass = ObjectFactory.findProviderClass(
+                    "org.apache.xpath.domapi.XPathEvaluatorImpl",
+                    ObjectFactory.findClassLoader(), true);
+                Constructor xpathClassConstr = 
+                    xpathClass.getConstructor(new Class[] { Document.class });
+                
+                // Check if the DOM XPath implementation implements
+                // the interface org.w3c.dom.XPathEvaluator
+                Class interfaces[] = xpathClass.getInterfaces();
+                for (int i = 0; i < interfaces.length; i++) {
+                    if (interfaces[i].getName().equals(
+                    "org.w3c.dom.xpath.XPathEvaluator")) {
+                        fXPathEvaluator = xpathClassConstr.newInstance(new Object[] { this });
+                        return fXPathEvaluator;
+                    }
+                }
+                return null;
+            } catch (Exception e) {
                 return null;
             }
         }
@@ -530,22 +532,22 @@ extends ParentNode implements Document  {
     //
     // Document methods
     //
-
+    
     // factory methods
-
+    
     /**
      * Factory method; creates an Attribute having this Document as its
      * OwnerDoc.
-     *
-     * @param name The name of the attribute. Note that the attribute's value
-     * is _not_ established at the factory; remember to set it!
-     *
-     * @throws DOMException(INVALID_NAME_ERR) if the attribute name is not
-     * acceptable.
+     * 
+     * @param name The name of the attribute. Note that the attribute's value is
+     * _not_ established at the factory; remember to set it!
+     * 
+     * @throws DOMException(INVALID_NAME_ERR)
+     * if the attribute name is not acceptable.
      */
     public Attr createAttribute(String name)
-    throws DOMException {
-
+        throws DOMException {
+        
         if (errorChecking && !isXMLName(name,xml11Version)) {
             String msg =
                 DOMMessageFormatter.formatMessage(
@@ -555,7 +557,7 @@ extends ParentNode implements Document  {
             throw new DOMException(DOMException.INVALID_CHARACTER_ERR, msg);
         }
         return new AttrImpl(this, name);
-
+        
     } // createAttribute(String):Attr
 
     /**
@@ -811,6 +813,15 @@ extends ParentNode implements Document  {
     public void setXmlEncoding(String value) {
         encoding = value;
     }
+    
+    /**
+     * @deprecated This method is internal and only exists for
+     * compatibility with older applications. New applications
+     * should never call this method.
+     */
+    public void setEncoding(String value) {
+        setXmlEncoding(value);
+    }
 
     /**
      * DOM Level 3 WD - Experimental.
@@ -819,6 +830,15 @@ extends ParentNode implements Document  {
     public String getXmlEncoding() {
         return encoding;
     }
+    
+    /**
+     * @deprecated This method is internal and only exists for
+     * compatibility with older applications. New applications
+     * should never call this method.
+     */
+    public String getEncoding() {
+        return getXmlEncoding();
+    }
 
     /**
      * DOM Level 3 CR - Experimental.
@@ -851,6 +871,15 @@ extends ParentNode implements Document  {
             xml11Version = false;
         }
     }
+    
+    /**
+     * @deprecated This method is internal and only exists for
+     * compatibility with older applications. New applications
+     * should never call this method.
+     */
+    public void setVersion(String value) {
+        setXmlVersion(value);
+    }
 
     /**
      * DOM Level 3 WD - Experimental.
@@ -859,6 +888,15 @@ extends ParentNode implements Document  {
     public String getXmlVersion() {
         return (version == null)?"1.0":version;
     }
+    
+    /**
+     * @deprecated This method is internal and only exists for
+     * compatibility with older applications. New applications
+     * should never call this method.
+     */
+    public String getVersion() {
+        return getXmlVersion();
+    }
 
     /**
      * DOM Level 3 CR - Experimental.
@@ -874,6 +912,15 @@ extends ParentNode implements Document  {
                                   throws DOMException {
             standalone = value;
     }
+    
+    /**
+     * @deprecated This method is internal and only exists for
+     * compatibility with older applications. New applications
+     * should never call this method.
+     */
+    public void setStandalone(boolean value) {
+        setXmlStandalone(value);
+    }
 
     /**
      * DOM Level 3 WD - Experimental.
@@ -883,7 +930,15 @@ extends ParentNode implements Document  {
     public boolean getXmlStandalone() {
         return standalone;
     }
-
+    
+    /**
+     * @deprecated This method is internal and only exists for
+     * compatibility with older applications. New applications
+     * should never call this method.
+     */
+    public boolean getStandalone() {
+        return getXmlStandalone();
+    }
 
     /**
      * DOM Level 3 WD - Experimental.
@@ -904,10 +959,10 @@ extends ParentNode implements Document  {
      */
     public Node renameNode(Node n,String namespaceURI,String name)
     throws DOMException{
-
-        if (n.getOwnerDocument() != this) {
+        
+        if (errorChecking && n.getOwnerDocument() != this && n != this) {
             String msg = DOMMessageFormatter.formatMessage(
-            DOMMessageFormatter.DOM_DOMAIN, "WRONG_DOCUMENT_ERR", null);
+                    DOMMessageFormatter.DOM_DOMAIN, "WRONG_DOCUMENT_ERR", null);
             throw new DOMException(DOMException.WRONG_DOCUMENT_ERR, msg);
         }
         switch (n.getNodeType()) {
@@ -915,38 +970,47 @@ extends ParentNode implements Document  {
                 ElementImpl el = (ElementImpl) n;
                 if (el instanceof ElementNSImpl) {
                     ((ElementNSImpl) el).rename(namespaceURI, name);
+                    
+                    // fire user data NODE_RENAMED event
+                    callUserDataHandlers(el, null, UserDataHandler.NODE_RENAMED);
                 }
                 else {
                     if (namespaceURI == null) {
-                        int colon1 = name.indexOf(':');
-                        if(colon1 != -1){
-                            String msg =
-                            DOMMessageFormatter.formatMessage(
-                            DOMMessageFormatter.DOM_DOMAIN,
-                            "NAMESPACE_ERR",
-                            null);
-                            throw new DOMException(DOMException.NAMESPACE_ERR, msg);
-                        }
-                        if (errorChecking && !isXMLName(name,xml11Version)) {
-                            String msg = DOMMessageFormatter.formatMessage(
-                            DOMMessageFormatter.DOM_DOMAIN,
-                            "INVALID_CHARACTER_ERR", null);
-                            throw new DOMException(DOMException.INVALID_CHARACTER_ERR,
-                            msg);
+                        if (errorChecking) {
+                            int colon1 = name.indexOf(':');
+                            if(colon1 != -1){
+                                String msg =
+                                    DOMMessageFormatter.formatMessage(
+                                            DOMMessageFormatter.DOM_DOMAIN,
+                                            "NAMESPACE_ERR",
+                                            null);
+                                throw new DOMException(DOMException.NAMESPACE_ERR, msg);
+                            }
+                            if (!isXMLName(name,xml11Version)) {
+                                String msg = DOMMessageFormatter.formatMessage(
+                                        DOMMessageFormatter.DOM_DOMAIN,
+                                        "INVALID_CHARACTER_ERR", null);
+                                throw new DOMException(DOMException.INVALID_CHARACTER_ERR,
+                                        msg);
+                            }
                         }
                         el.rename(name);
+                        
+                        // fire user data NODE_RENAMED event
+                        callUserDataHandlers(el, null,
+                                UserDataHandler.NODE_RENAMED);
                     }
                     else {
                         // we need to create a new object
                         ElementNSImpl nel =
-                        new ElementNSImpl(this, namespaceURI, name);
-
+                            new ElementNSImpl(this, namespaceURI, name);
+                        
                         // register event listeners on new node
                         copyEventListeners(el, nel);
-
+                        
                         // remove user data from old node
                         Hashtable data = removeUserDataTable(el);
-
+                        
                         // remove old node from parent if any
                         Node parent = el.getParentNode();
                         Node nextSib = el.getNextSibling();
@@ -962,14 +1026,14 @@ extends ParentNode implements Document  {
                         }
                         // move specified attributes to new node
                         nel.moveSpecifiedAttributes(el);
-
+                        
                         // attach user data to new node
                         setUserDataTable(nel, data);
-
+                        
                         // and fire user data NODE_RENAMED event
                         callUserDataHandlers(el, nel,
-                        UserDataHandler.NODE_RENAMED);
-
+                                UserDataHandler.NODE_RENAMED);
+                        
                         // insert new node where old one was
                         if (parent != null) {
                             parent.insertBefore(nel, nextSib);
@@ -983,7 +1047,7 @@ extends ParentNode implements Document  {
             }
             case ATTRIBUTE_NODE: {
                 AttrImpl at = (AttrImpl) n;
-
+                
                 // dettach attr from element
                 Element el = at.getOwnerElement();
                 if (el != null) {
@@ -995,6 +1059,9 @@ extends ParentNode implements Document  {
                     if (el != null) {
                         el.setAttributeNodeNS(at);
                     }
+                    
+                    // fire user data NODE_RENAMED event
+                    callUserDataHandlers(at, null, UserDataHandler.NODE_RENAMED);
                 }
                 else {
                     if (namespaceURI == null) {
@@ -1003,18 +1070,20 @@ extends ParentNode implements Document  {
                         if (el != null) {
                             el.setAttributeNode(at);
                         }
+                        
+                        // fire user data NODE_RENAMED event
+                        callUserDataHandlers(at, null, UserDataHandler.NODE_RENAMED);
                     }
                     else {
                         // we need to create a new object
-                        AttrNSImpl nat =
-                        new AttrNSImpl(this, namespaceURI, name);
-
+                        AttrNSImpl nat = new AttrNSImpl(this, namespaceURI, name);
+                        
                         // register event listeners on new node
                         copyEventListeners(at, nat);
-
+                        
                         // remove user data from old node
                         Hashtable data = removeUserDataTable(at);
-
+                        
                         // move children to new node
                         Node child = at.getFirstChild();
                         while (child != null) {
@@ -1022,14 +1091,13 @@ extends ParentNode implements Document  {
                             nat.appendChild(child);
                             child = at.getFirstChild();
                         }
-
+                        
                         // attach user data to new node
                         setUserDataTable(nat, data);
-
+                        
                         // and fire user data NODE_RENAMED event
-                        callUserDataHandlers(at, nat,
-                        UserDataHandler.NODE_RENAMED);
-
+                        callUserDataHandlers(at, nat, UserDataHandler.NODE_RENAMED);
+                        
                         // reattach attr to element
                         if (el != null) {
                             el.setAttributeNode(nat);
@@ -1039,7 +1107,7 @@ extends ParentNode implements Document  {
                 }
                 // fire AttributeNameChanged event
                 renamedAttrNode((Attr) n, at);
-
+                
                 return at;
             }
             default: {
@@ -1047,7 +1115,7 @@ extends ParentNode implements Document  {
                 throw new DOMException(DOMException.NOT_SUPPORTED_ERR, msg);
             }
         }
-
+        
     }
 
 
@@ -1099,10 +1167,23 @@ extends ParentNode implements Document  {
 
 
     /**
-     * DOM Level 3 WD - Experimental.
-     * Retrieve baseURI
+     * Returns the absolute base URI of this node or null if the implementation
+     * wasn't able to obtain an absolute URI. Note: If the URI is malformed, a
+     * null is returned.
+     * 
+     * @return The absolute base URI of this node or null.
+     * @since DOM Level 3
      */
     public String getBaseURI() {
+        if (fDocumentURI != null && fDocumentURI.length() != 0 ) {// attribute value is always empty string
+            try {
+                return new URI(fDocumentURI).toString();
+            }
+            catch (org.apache.xerces.util.URI.MalformedURIException e){
+                // REVISIT: what should happen in this case?
+                return null;
+            }
+        }            
         return fDocumentURI;
     }
 
@@ -1237,8 +1318,8 @@ extends ParentNode implements Document  {
      */
     public String saveXML(Node node)
     throws DOMException {
-        if ( node != null &&
-        this != node.getOwnerDocument() ) {
+        if ( errorChecking && node != null &&
+            this != node.getOwnerDocument() ) {
             String msg = DOMMessageFormatter.formatMessage(DOMMessageFormatter.DOM_DOMAIN, "WRONG_DOCUMENT_ERR", null);
             throw new DOMException(DOMException.WRONG_DOCUMENT_ERR, msg);
         }
@@ -1250,7 +1331,6 @@ extends ParentNode implements Document  {
         return xmlWriter.writeToString(node);
     }
 
-
     /**
      * Sets whether the DOM implementation generates mutation events
      * upon operations.
@@ -1426,6 +1506,7 @@ extends ParentNode implements Document  {
     Hashtable reversedIdentifiers)
     throws DOMException {
         Node newnode=null;
+		Hashtable userData = null;
 
         // Sigh. This doesn't work; too many nodes have private data that
         // would have to be manually tweaked. May be able to add local
@@ -1438,7 +1519,8 @@ extends ParentNode implements Document  {
         //  newnode.ownerDocument=this;
         // }
         // else
-
+		if(source instanceof NodeImpl)
+			userData = ((NodeImpl)source).getUserDataRecord();
         int type = source.getNodeType();
         switch (type) {
             case ELEMENT_NODE: {
@@ -1590,6 +1672,7 @@ extends ParentNode implements Document  {
                 createDocumentType(srcdoctype.getNodeName(),
                 srcdoctype.getPublicId(),
                 srcdoctype.getSystemId());
+                newdoctype.setInternalSubset(srcdoctype.getInternalSubset());
                 // Values are on NamedNodeMaps
                 NamedNodeMap smap = srcdoctype.getEntities();
                 NamedNodeMap tmap = newdoctype.getEntities();
@@ -1641,7 +1724,8 @@ extends ParentNode implements Document  {
             }
         }
 
-        callUserDataHandlers(source, newnode, UserDataHandler.NODE_IMPORTED);
+		if(userData != null)
+			callUserDataHandlers(source, newnode, UserDataHandler.NODE_IMPORTED,userData);
 
         // If deep, replicate and attach the kids.
         if (deep) {
@@ -1668,12 +1752,41 @@ extends ParentNode implements Document  {
      **/
     public Node adoptNode(Node source) {
         NodeImpl node;
+		Hashtable userData = null;
         try {
             node = (NodeImpl) source;
         } catch (ClassCastException e) {
             // source node comes from a different DOMImplementation
             return null;
         }
+        
+        // Return null if the source is null
+        
+        if (source == null ) {
+        	return null;
+        } else if (source != null && source.getOwnerDocument() != null) {
+
+            DOMImplementation thisImpl = this.getImplementation();
+            DOMImplementation otherImpl = source.getOwnerDocument().getImplementation();
+            
+            // when the source node comes from a different implementation.
+            if (thisImpl != otherImpl) {
+            
+                // Adopting from a DefferedDOM to DOM
+                if (thisImpl instanceof org.apache.xerces.dom.DOMImplementationImpl &&
+                        otherImpl instanceof org.apache.xerces.dom.DeferredDOMImplementationImpl) {
+                    // traverse the DOM and expand deffered nodes and then allow adoption
+                    undeferChildren (node);
+                } else if ( thisImpl instanceof org.apache.xerces.dom.DeferredDOMImplementationImpl
+                        && otherImpl instanceof org.apache.xerces.dom.DOMImplementationImpl) {
+                    // Adopting from a DOM into a DefferedDOM, this should be okay
+                } else {
+                    // Adopting between two dissimilar DOM's is not allowed
+                    return null;  
+                }
+        	}
+        }
+        
         switch (node.getNodeType()) {
             case ATTRIBUTE_NODE: {
                 AttrImpl attr = (AttrImpl) node;
@@ -1684,9 +1797,12 @@ extends ParentNode implements Document  {
                 }
                 //2. specified flag is set to true
                 attr.isSpecified(true);
+				userData = node.getUserDataRecord();
 
                 //3. change ownership
                 attr.setOwnerDocument(this);
+				if(userData != null )
+					setUserDataTable(node,userData);
                 break;
             }
             //entity, notation nodes are read only nodes.. so they can't be adopted.
@@ -1705,6 +1821,7 @@ extends ParentNode implements Document  {
                 throw new DOMException(DOMException.NOT_SUPPORTED_ERR, msg);
             }
             case ENTITY_REFERENCE_NODE: {
+				userData = node.getUserDataRecord();
                 // remove node from wherever it is
                 Node parent = node.getParentNode();
                 if (parent != null) {
@@ -1717,6 +1834,8 @@ extends ParentNode implements Document  {
                 }
                 // change ownership
                 node.setOwnerDocument(this);
+				if(userData != null)
+					setUserDataTable(node,userData);
                 // set its new replacement value if any
                 if (docType == null) {
                     break;
@@ -1726,7 +1845,6 @@ extends ParentNode implements Document  {
                 if (entityNode == null) {
                     break;
                 }
-                EntityImpl entity = (EntityImpl) entityNode;
                 for (child = entityNode.getFirstChild();
                 child != null; child = child.getNextSibling()) {
                     Node childClone = child.cloneNode(true);
@@ -1735,6 +1853,7 @@ extends ParentNode implements Document  {
                 break;
             }
             case ELEMENT_NODE: {
+				userData = node.getUserDataRecord();
                 // remove node from wherever it is
                 Node parent = node.getParentNode();
                 if (parent != null) {
@@ -1742,11 +1861,14 @@ extends ParentNode implements Document  {
                 }
                 // change ownership
                 node.setOwnerDocument(this);
+				if(userData != null)
+					setUserDataTable(node,userData);
                 // reconcile default attributes
                 ((ElementImpl)node).reconcileDefaultAttributes();
                 break;
             }
             default: {
+				userData = node.getUserDataRecord();
                 // remove node from wherever it is
                 Node parent = node.getParentNode();
                 if (parent != null) {
@@ -1754,16 +1876,66 @@ extends ParentNode implements Document  {
                 }
                 // change ownership
                 node.setOwnerDocument(this);
+				if(userData != null)
+					setUserDataTable(node,userData);
             }
         }
 
 		//DOM L3 Core CR
 		//http://www.w3.org/TR/2003/CR-DOM-Level-3-Core-20031107/core.html#UserDataHandler-ADOPTED
-		callUserDataHandlers(source, null, UserDataHandler.NODE_ADOPTED);
+		if(userData != null)
+			callUserDataHandlers(source, null, UserDataHandler.NODE_ADOPTED,userData);
 
         return node;
     }
 
+    /**
+     * Traverses the DOM Tree and expands deferred nodes and their
+     * children.
+     * 
+     */
+    protected void undeferChildren(Node node) {
+        
+        Node top = node;
+        
+        while (null != node) {
+            
+            if (((NodeImpl)node).needsSyncData()) {
+                ((NodeImpl)node).synchronizeData();
+            }
+            
+            NamedNodeMap attributes = node.getAttributes();
+            if (attributes != null) {
+                int length = attributes.getLength();
+                for (int i = 0; i < length; ++i) {
+                    undeferChildren(attributes.item(i));
+                }
+            }
+            
+            Node nextNode = null;
+            nextNode = node.getFirstChild();
+            
+            while (null == nextNode) {
+                
+                if (top.equals(node))
+                    break;
+                
+                nextNode = node.getNextSibling();
+                
+                if (null == nextNode) {
+                    node = node.getParentNode();
+                    
+                    if ((null == node) || (top.equals(node))) {
+                        nextNode = null;
+                        break;
+                    }
+                }
+            }
+            
+            node = nextNode;
+        }
+    }
+    
     // identifier maintenence
     /**
      * Introduced in DOM Level 2
@@ -2122,17 +2294,6 @@ extends ParentNode implements Document  {
     }
 
 
-    /*
-     * a class to store some user data along with its handler
-     */
-    class UserDataRecord implements Serializable {
-        Object fData;
-        UserDataHandler fHandler;
-        UserDataRecord(Object data, UserDataHandler handler) {
-            fData = data;
-            fHandler = handler;
-        }
-    }
 
     /**
      * Associate an object to a key on this node. The object can later be
@@ -2187,6 +2348,7 @@ extends ParentNode implements Document  {
             return null;
         }
     }
+	
 
     /**
      * Retrieves the object associated to a key on a this node. The object
@@ -2214,7 +2376,18 @@ extends ParentNode implements Document  {
         return null;
     }
 
-    /**
+	protected Hashtable getUserDataRecord(Node n){
+        if (userData == null) {
+            return null;
+        }
+        Hashtable t = (Hashtable) userData.get(n);
+        if (t == null) {
+            return null;
+        }
+		return t;
+	}
+    
+	/**
      * Remove user data table for the given node.
      * @param n The node this operation applies to.
      * @return The removed table.
@@ -2232,6 +2405,8 @@ extends ParentNode implements Document  {
      * @param data The user data table.
      */
     void setUserDataTable(Node n, Hashtable data) {
+		if (userData == null)
+			userData = new Hashtable();
         if (data != null) {
             userData.put(n, data);
         }
@@ -2243,25 +2418,42 @@ extends ParentNode implements Document  {
      * @param c The copy node or null.
      * @param operation The operation - import, clone, or delete.
      */
-    void callUserDataHandlers(Node n, Node c, short operation) {
+    protected void callUserDataHandlers(Node n, Node c, short operation) {
         if (userData == null) {
             return;
         }
-        Hashtable t = (Hashtable) userData.get(n);
-        if (t == null || t.isEmpty()) {
+        //Hashtable t = (Hashtable) userData.get(n);
+		if(n instanceof NodeImpl){
+			Hashtable t = ((NodeImpl)n).getUserDataRecord();
+			if (t == null || t.isEmpty()) {
+				return;
+			}
+			callUserDataHandlers(n, c, operation,t);
+		}
+    }
+
+	/**
+     * Call user data handlers when a node is deleted (finalized)
+     * @param n The node this operation applies to.
+     * @param c The copy node or null.
+     * @param operation The operation - import, clone, or delete.
+	 * @param handlers Data associated with n.
+	*/
+	void callUserDataHandlers(Node n, Node c, short operation,Hashtable userData) {
+        if (userData == null || userData.isEmpty()) {
             return;
         }
-        Enumeration keys = t.keys();
+        Enumeration keys = userData.keys();
         while (keys.hasMoreElements()) {
             String key = (String) keys.nextElement();
-            UserDataRecord r = (UserDataRecord) t.get(key);
+            UserDataRecord r = (UserDataRecord) userData.get(key);
             if (r.fHandler != null) {
                 r.fHandler.handle(operation, key, r.fData, n, c);
             }
         }
     }
-
-    /**
+    
+	/**
      * Call user data handlers to let them know the nodes they are related to
      * are being deleted. The alternative would be to do that on Node but
      * because the nodes are used as the keys we have a reference to them that
@@ -2470,15 +2662,15 @@ extends ParentNode implements Document  {
     }
 
     /**
-     * A method to be called when a character data node has been modified
+     * A method to be called when a character data node is about to be modified
      */
-    void modifyingCharacterData(NodeImpl node) {
+    void modifyingCharacterData(NodeImpl node, boolean replace) {
     }
 
     /**
      * A method to be called when a character data node has been modified
      */
-    void modifiedCharacterData(NodeImpl node, String oldvalue, String value) {
+    void modifiedCharacterData(NodeImpl node, String oldvalue, String value, boolean replace) {
     }
 
     /**
@@ -2517,6 +2709,19 @@ extends ParentNode implements Document  {
     void replacedNode(NodeImpl node) {
     }
 
+    /**
+     * A method to be called when a character data node is about to be replaced
+     */
+    void replacingData(NodeImpl node) {
+    }
+
+    /**
+     *  method to be called when a character data node has been replaced.
+     */
+    void replacedCharacterData(NodeImpl node, String oldvalue, String value) {
+    }
+
+    
     /**
      * A method to be called when an attribute value has been modified
      */
