@@ -1,58 +1,17 @@
 /*
- * The Apache Software License, Version 1.1
- *
- *
- * Copyright (c) 1999-2004 The Apache Software Foundation.  
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.apache.org.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2004,2006 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package org.apache.xerces.impl;
@@ -61,9 +20,9 @@ import java.io.IOException;
 
 import org.apache.xerces.impl.msg.XMLMessageFormatter;
 import org.apache.xerces.util.SymbolTable;
-import org.apache.xerces.util.XMLAttributesImpl;
 import org.apache.xerces.util.XMLChar;
 import org.apache.xerces.util.XMLStringBuffer;
+import org.apache.xerces.xni.Augmentations;
 import org.apache.xerces.xni.XMLDTDContentModelHandler;
 import org.apache.xerces.xni.XMLDTDHandler;
 import org.apache.xerces.xni.XMLResourceIdentifier;
@@ -90,13 +49,15 @@ import org.apache.xerces.xni.parser.XMLInputSource;
  *  <li>http://apache.org/xml/properties/internal/error-reporter</li>
  *  <li>http://apache.org/xml/properties/internal/entity-manager</li>
  * </ul>
+ * 
+ * @xerces.internal
  *
  * @author Arnaud  Le Hors, IBM
  * @author Andy Clark, IBM
  * @author Glenn Marcy, IBM
  * @author Eric Ye, IBM
  *
- * @version $Id: XMLDTDScannerImpl.java,v 1.47 2004/02/11 18:55:20 mrglavas Exp $
+ * @version $Id: XMLDTDScannerImpl.java 377620 2006-02-14 04:56:13Z mrglavas $
  */
 public class XMLDTDScannerImpl
     extends XMLScanner
@@ -181,9 +142,6 @@ public class XMLDTDScannerImpl
     /** Start DTD called. */
     private boolean fStartDTDCalled;
 
-    /** Default attribute */
-    private XMLAttributesImpl fAttributes = new XMLAttributesImpl();
-
     /** 
      * Stack of content operators (either '|' or ',') in children 
      * content.
@@ -517,14 +475,15 @@ public class XMLDTDScannerImpl
      *                 where the entity encoding is not auto-detected (e.g.
      *                 internal entities or a document entity that is
      *                 parsed from a java.io.Reader).
+     * @param augs     Additional information that may include infoset augmentations
      *
      * @throws XNIException Thrown by handler to signal an error.
      */
     public void startEntity(String name, 
                             XMLResourceIdentifier identifier,
-                            String encoding) throws XNIException {
+                            String encoding, Augmentations augs) throws XNIException {
 
-        super.startEntity(name, identifier, encoding);
+        super.startEntity(name, identifier, encoding, augs);
 
         boolean dtdEntity = name.equals("[dtd]");
         if (dtdEntity) {
@@ -547,7 +506,7 @@ public class XMLDTDScannerImpl
 
         // call handler
         if (fDTDHandler != null && !dtdEntity && fReportEntity) {
-            fDTDHandler.startParameterEntity(name, identifier, encoding, null);
+            fDTDHandler.startParameterEntity(name, identifier, encoding, augs);
         }
 
     } // startEntity(String,XMLResourceIdentifier,String)
@@ -558,13 +517,14 @@ public class XMLDTDScannerImpl
      * are just specified by their name.
      * 
      * @param name The name of the entity.
+     * @param augs Additional information that may include infoset augmentations
      *
      * @throws XNIException Thrown by handler to signal an error.
      */
-    public void endEntity(String name) 
+    public void endEntity(String name, Augmentations augs) 
         throws XNIException {
 
-        super.endEntity(name);
+        super.endEntity(name, augs);
 
         // if there is no data after the doctype
         //  
@@ -600,16 +560,13 @@ public class XMLDTDScannerImpl
             if (fEntityScanner.isExternal()) {
                 fExtEntityDepth--;
             }
+            // call handler
+            if (fDTDHandler != null && reportEntity) {
+                fDTDHandler.endParameterEntity(name, augs);
+            }
         }
-
-        // call handler
-        boolean dtdEntity = name.equals("[dtd]");
-        if (fDTDHandler != null && !dtdEntity && reportEntity) {
-            fDTDHandler.endParameterEntity(name, null);
-        }
-
         // end DTD
-        if (dtdEntity) {
+        else if (name.equals("[dtd]")) {
             if (fIncludeSectDepth != 0) {
                 reportFatalError("IncludeSectUnterminated", null);
             }
@@ -743,7 +700,10 @@ public class XMLDTDScannerImpl
                 version = fStrings[0];
                 encoding = fStrings[1];
 
-                fEntityScanner.setEncoding(encoding);
+                fEntityScanner.setXMLVersion(version);
+                if (!fEntityScanner.fCurrentEntity.isEncodingExternallySpecified()) {
+                    fEntityScanner.setEncoding(encoding);
+                }
 
                 // call handler
                 if (fDTDHandler != null) {
