@@ -1,58 +1,17 @@
 /*
- * The Apache Software License, Version 1.1
- *
- *
- * Copyright (c) 1999-2002 The Apache Software Foundation.
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 2002, International
- * Business Machines, Inc., http://www.apache.org.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2002,2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package org.apache.xerces.impl;
@@ -96,10 +55,12 @@ import org.apache.xerces.xni.parser.XMLDocumentSource;
  *  <li>http://apache.org/xml/properties/internal/entity-manager</li>
  *  <li>http://apache.org/xml/properties/internal/dtd-scanner</li>
  * </ul>
+ * 
+ * @xerces.internal
  *
  * @author Elena Litani, IBM
  *
- * @version $Id: XMLNSDocumentScannerImpl.java,v 1.20 2004/02/19 21:47:17 mrglavas Exp $
+ * @version $Id: XMLNSDocumentScannerImpl.java 320089 2004-10-04 21:45:49Z mrglavas $
  */
 public class XMLNSDocumentScannerImpl
 extends XMLDocumentScannerImpl {
@@ -112,16 +73,20 @@ extends XMLDocumentScannerImpl {
       *   scanner if DTD grammar is missing.*/
     protected boolean fPerformValidation;
 
-    // protected String[] fUri= new String[4];
-    // protected String[] fLocalpart = new String[4];
-    // protected int fLength = 0;
-
-
     // private data
     //
 
     /** DTD validator */
     private XMLDTDValidatorFilter fDTDValidator;
+    
+    /** 
+     * Saw spaces after element name or between attributes.
+     * 
+     * This is reserved for the case where scanning of a start element spans
+     * several methods, as is the case when scanning the start of a root element 
+     * where a DTD external subset may be read after scanning the element name.
+     */
+    private boolean fSawSpace;
 
 
     /**
@@ -326,53 +291,208 @@ extends XMLDocumentScannerImpl {
 
     } // scanStartElement():boolean
 
-    /** private final void checkDuplicates(QName qname, XMLAttributesImpl attributes){
-
-        // Example: <foo xmlns:a='NS' xmlns:b='NS' a:attr='v1' b:attr='v2'/>
-        // search if such attribute already exists
-        for (int i = 0; i < fLength; i++) {
-            if (qname.uri == fUri[i] && fLocalpart[i].equals(qname.localpart)) {
-                fErrorReporter.reportError(XMLMessageFormatter.XMLNS_DOMAIN,
-                                           "AttributeNSNotUnique",
-                                           new Object[]{fElementQName.rawname,qname.rawname, qname.uri},
-                                           XMLErrorReporter.SEVERITY_FATAL_ERROR);
-            }
-        }
-        int index = fLength;
-        if (fLength++ == fUri.length) {
-            String[] uris = new String[fUri.length + 4];
-            String[] lps = new String [fUri.length + 4];
-            System.arraycopy(fUri, 0, uris, 0, fUri.length);
-            System.arraycopy(fLocalpart, 0, lps, 0, fLocalpart.length);
-            fUri = uris;
-            fLocalpart = lps;
-
-        }
-
-        fUri[index] = qname.uri;
-        fLocalpart[index] = qname.localpart;
-    } **/
-
-
-
+    /**
+     * Scans the name of an element in a start or empty tag. 
+     * 
+     * @see #scanStartElement()
+     */
+    protected void scanStartElementName ()
+        throws IOException, XNIException {
+        // Note: namespace processing is on by default
+        fEntityScanner.scanQName(fElementQName);
+        // Must skip spaces here because the DTD scanner
+        // would consume them at the end of the external subset.
+        fSawSpace = fEntityScanner.skipSpaces();
+    } // scanStartElementName()
 
     /**
- * Scans an attribute.
- * <p>
- * <pre>
- * [41] Attribute ::= Name Eq AttValue
- * </pre>
- * <p>
- * <strong>Note:</strong> This method assumes that the next
- * character on the stream is the first character of the attribute
- * name.
- * <p>
- * <strong>Note:</strong> This method uses the fAttributeQName and
- * fQName variables. The contents of these variables will be
- * destroyed.
- *
- * @param attributes The attributes list for the scanned attribute.
- */
+     * Scans the remainder of a start or empty tag after the element name.
+     * 
+     * @see #scanStartElement
+     * @return True if element is empty.
+     */
+    protected boolean scanStartElementAfterName()
+        throws IOException, XNIException {
+    	
+        // REVISIT - [Q] Why do we need this temp variable? -- mrglavas
+        String rawname = fElementQName.rawname;
+        if (fBindNamespaces) {
+            fNamespaceContext.pushContext();
+            if (fScannerState == SCANNER_STATE_ROOT_ELEMENT) {
+                if (fPerformValidation) {
+                    fErrorReporter.reportError(XMLMessageFormatter.XML_DOMAIN,
+                                               "MSG_GRAMMAR_NOT_FOUND",
+                                               new Object[]{ rawname},
+                                               XMLErrorReporter.SEVERITY_ERROR);
+
+                    if (fDoctypeName == null || !fDoctypeName.equals(rawname)) {
+                        fErrorReporter.reportError( XMLMessageFormatter.XML_DOMAIN,
+                                                    "RootElementTypeMustMatchDoctypedecl",
+                                                    new Object[]{fDoctypeName, rawname},
+                                                    XMLErrorReporter.SEVERITY_ERROR);
+                    }
+                }
+            }
+        }
+
+        // push element stack
+        fCurrentElement = fElementStack.pushElement(fElementQName);
+
+        // attributes
+        boolean empty = false;
+        fAttributes.removeAllAttributes();
+        do {
+        	
+            // end tag?
+            int c = fEntityScanner.peekChar();
+            if (c == '>') {
+                fEntityScanner.scanChar();
+                break;
+            }
+            else if (c == '/') {
+                fEntityScanner.scanChar();
+                if (!fEntityScanner.skipChar('>')) {
+                    reportFatalError("ElementUnterminated",
+                                     new Object[]{rawname});
+                }
+                empty = true;
+                break;
+            }
+            else if (!isValidNameStartChar(c) || !fSawSpace) {
+                reportFatalError("ElementUnterminated", new Object[]{rawname});
+            }
+
+            // attributes
+            scanAttribute(fAttributes);
+            
+            // spaces
+            fSawSpace = fEntityScanner.skipSpaces();
+
+        } while (true);
+
+        if (fBindNamespaces) {
+            // REVISIT: is it required? forbit xmlns prefix for element
+            if (fElementQName.prefix == XMLSymbols.PREFIX_XMLNS) {
+                fErrorReporter.reportError(XMLMessageFormatter.XMLNS_DOMAIN,
+                                           "ElementXMLNSPrefix",
+                                           new Object[]{fElementQName.rawname},
+                                           XMLErrorReporter.SEVERITY_FATAL_ERROR);
+            }
+
+            // bind the element
+            String prefix = fElementQName.prefix != null
+                            ? fElementQName.prefix : XMLSymbols.EMPTY_STRING;
+            // assign uri to the element
+            fElementQName.uri = fNamespaceContext.getURI(prefix);
+            // make sure that object in the element stack is updated as well
+            fCurrentElement.uri = fElementQName.uri;
+
+            if (fElementQName.prefix == null && fElementQName.uri != null) {
+                fElementQName.prefix = XMLSymbols.EMPTY_STRING;
+                // making sure that the object in the element stack is updated too.
+                fCurrentElement.prefix = XMLSymbols.EMPTY_STRING;
+            }
+            if (fElementQName.prefix != null && fElementQName.uri == null) {
+                fErrorReporter.reportError(XMLMessageFormatter.XMLNS_DOMAIN,
+                                           "ElementPrefixUnbound",
+                                           new Object[]{fElementQName.prefix, fElementQName.rawname},
+                                           XMLErrorReporter.SEVERITY_FATAL_ERROR);
+            }
+
+            // bind attributes (xmlns are already bound bellow)
+            int length = fAttributes.getLength();
+            // fLength = 0; //initialize structure
+            for (int i = 0; i < length; i++) {
+                fAttributes.getName(i, fAttributeQName);
+
+                String aprefix = fAttributeQName.prefix != null
+                                 ? fAttributeQName.prefix : XMLSymbols.EMPTY_STRING;
+                String uri = fNamespaceContext.getURI(aprefix);
+                // REVISIT: try removing the first "if" and see if it is faster.
+                //
+                if (fAttributeQName.uri != null && fAttributeQName.uri == uri) {
+                    // checkDuplicates(fAttributeQName, fAttributes);
+                    continue;
+                }
+                if (aprefix != XMLSymbols.EMPTY_STRING) {
+                    fAttributeQName.uri = uri;
+                    if (uri == null) {
+                        fErrorReporter.reportError(XMLMessageFormatter.XMLNS_DOMAIN,
+                                                   "AttributePrefixUnbound",
+                                                   new Object[]{fElementQName.rawname,fAttributeQName.rawname,aprefix},
+                                                   XMLErrorReporter.SEVERITY_FATAL_ERROR);
+                    }
+                    fAttributes.setURI(i, uri);
+                    // checkDuplicates(fAttributeQName, fAttributes);
+                }
+            }
+            
+            if (length > 1) {
+                QName name = fAttributes.checkDuplicatesNS();
+                if (name != null) {
+                    if (name.uri != null) {
+                        fErrorReporter.reportError(XMLMessageFormatter.XMLNS_DOMAIN,
+                                                   "AttributeNSNotUnique",
+                                                   new Object[]{fElementQName.rawname, name.localpart, name.uri},
+                                                   XMLErrorReporter.SEVERITY_FATAL_ERROR);
+                    }
+                    else {
+                        fErrorReporter.reportError(XMLMessageFormatter.XMLNS_DOMAIN,
+                                                   "AttributeNotUnique",
+                                                   new Object[]{fElementQName.rawname, name.rawname}, 
+                                                   XMLErrorReporter.SEVERITY_FATAL_ERROR);
+                    }
+                }
+            }
+        }
+
+
+        // call handler
+        if (fDocumentHandler != null) {
+            if (empty) {
+
+                //decrease the markup depth..
+                fMarkupDepth--;
+
+                // check that this element was opened in the same entity
+                if (fMarkupDepth < fEntityStack[fEntityDepth - 1]) {
+                    reportFatalError("ElementEntityMismatch",
+                                     new Object[]{fCurrentElement.rawname});
+                }
+
+                fDocumentHandler.emptyElement(fElementQName, fAttributes, null);
+
+                if (fBindNamespaces) {
+                    fNamespaceContext.popContext();
+                }
+                //pop the element off the stack..
+                fElementStack.popElement(fElementQName);
+            } else {
+                fDocumentHandler.startElement(fElementQName, fAttributes, null);
+            }
+        }
+
+        if (DEBUG_CONTENT_SCANNING) System.out.println("<<< scanStartElementAfterName(): "+empty);
+        return empty;
+    } // scanStartElementAfterName()
+
+    /**
+     * Scans an attribute.
+     * <p>
+     * <pre>
+     * [41] Attribute ::= Name Eq AttValue
+     * </pre>
+     * <p>
+     * <strong>Note:</strong> This method assumes that the next
+     * character on the stream is the first character of the attribute
+     * name.
+     * <p>
+     * <strong>Note:</strong> This method uses the fAttributeQName and
+     * fQName variables. The contents of these variables will be
+     * destroyed.
+     *
+     * @param attributes The attributes list for the scanned attribute.
+     */
     protected void scanAttribute(XMLAttributesImpl attributes)
     throws IOException, XNIException {
         if (DEBUG_CONTENT_SCANNING) System.out.println(">>> scanAttribute()");
@@ -410,12 +530,16 @@ extends XMLDocumentScannerImpl {
         //REVISIT: one more case needs to be included: external PE and standalone is no
         boolean isVC =  fHasExternalDTD && !fStandalone;
 
-        scanAttributeValue(this.fTempString, fTempString2,
-                           fAttributeQName.rawname, isVC, 
-                           fCurrentElement.rawname);
+        // Scan attribute value and return true if the non-normalized and normalized value are the same
+        boolean isSameNormalizedAttr = scanAttributeValue(this.fTempString, fTempString2,
+                fAttributeQName.rawname, isVC, fCurrentElement.rawname);
+        
         String value = fTempString.toString();
         attributes.setValue(attrIndex, value);
-        attributes.setNonNormalizedValue(attrIndex, fTempString2.toString());
+        // If the non-normalized and normalized value are the same, avoid creating a new string.
+        if (!isSameNormalizedAttr) {
+            attributes.setNonNormalizedValue(attrIndex, fTempString2.toString());
+        }
         attributes.setSpecified(attrIndex, true);
 
         // record namespace declarations if any.
@@ -580,7 +704,8 @@ extends XMLDocumentScannerImpl {
      * Dispatcher to handle content scanning.
      */
     protected final class NSContentDispatcher
-    extends ContentDispatcher {
+        extends ContentDispatcher {
+        
         /**
          * Scan for root element hook. This method is a hook for
          * subclasses to add code that handles scanning for the root
@@ -595,7 +720,38 @@ extends XMLDocumentScannerImpl {
          *          the content dispatcher should continue as normal.
          */
         protected boolean scanRootElementHook()
-        throws IOException, XNIException {
+            throws IOException, XNIException {
+       
+            if (fExternalSubsetResolver != null && !fSeenDoctypeDecl
+                && !fDisallowDoctype && (fValidation || fLoadExternalDTD)) {
+                scanStartElementName();
+                resolveExternalSubsetAndRead();
+                reconfigurePipeline();
+                if (scanStartElementAfterName()) {
+                    setScannerState(SCANNER_STATE_TRAILING_MISC);
+                    setDispatcher(fTrailingMiscDispatcher);
+                    return true;
+                }
+            }
+            else {
+                reconfigurePipeline();
+                if (scanStartElement()) {
+                    setScannerState(SCANNER_STATE_TRAILING_MISC);
+                    setDispatcher(fTrailingMiscDispatcher);
+                    return true;
+                }
+            }
+            return false;
+
+        } // scanRootElementHook():boolean
+        
+        /**
+         * Re-configures pipeline by removing the DTD validator 
+         * if no DTD grammar exists. If no validator exists in the
+         * pipeline or there is no DTD grammar, namespace binding
+         * is performed by the scanner in the enclosing class.
+         */
+        private void reconfigurePipeline() {
             if (fDTDValidator == null) {
                 fBindNamespaces = true;
             }
@@ -611,15 +767,7 @@ extends XMLDocumentScannerImpl {
                 fDTDValidator.setDocumentSource(null);
                 fDTDValidator.setDocumentHandler(null);
             }
-
-            if (scanStartElement()) {
-                setScannerState(SCANNER_STATE_TRAILING_MISC);
-                setDispatcher(fTrailingMiscDispatcher);
-                return true;
-            }
-            return false;
-
-        } // scanRootElementHook():boolean
+        } // reconfigurePipeline()
     }
 
 } // class XMLNSDocumentScannerImpl
