@@ -1,58 +1,17 @@
 /*
- * The Apache Software License, Version 1.1
- *
- *
- * Copyright (c) 1999-2004 The Apache Software Foundation.  
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.apache.org.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package org.apache.xerces.impl;
@@ -64,7 +23,7 @@ import org.apache.xerces.util.SymbolTable;
 import org.apache.xerces.util.XMLChar;
 import org.apache.xerces.util.XMLResourceIdentifierImpl;
 import org.apache.xerces.util.XMLStringBuffer;
-import org.apache.xerces.xni.XMLAttributes;
+import org.apache.xerces.xni.Augmentations;
 import org.apache.xerces.xni.XMLResourceIdentifier;
 import org.apache.xerces.xni.XMLString;
 import org.apache.xerces.xni.XNIException;
@@ -89,12 +48,14 @@ import org.apache.xerces.xni.parser.XMLConfigurationException;
  *  <li>http://apache.org/xml/properties/internal/error-reporter</li>
  *  <li>http://apache.org/xml/properties/internal/entity-manager</li>
  * </ul>
+ * 
+ * @xerces.internal
  *
  * @author Andy Clark, IBM
  * @author Arnaud  Le Hors, IBM
  * @author Eric Ye, IBM
  *
- * @version $Id: XMLScanner.java,v 1.45 2004/02/11 18:55:20 mrglavas Exp $
+ * @version $Id: XMLScanner.java 320089 2004-10-04 21:45:49Z mrglavas $
  */
 public abstract class XMLScanner 
     implements XMLComponent {
@@ -787,10 +748,12 @@ public abstract class XMLScanner
      *                      false if undeclared entities should be reported as WFC violation.
      * @param eleName The name of element to which this attribute belongs.
      *
+     * @return true if the non-normalized and normalized value are the same
+     * 
      * <strong>Note:</strong> This method uses fStringBuffer2, anything in it
      * at the time of calling is lost.
      **/
-    protected void scanAttributeValue(XMLString value, 
+    protected boolean scanAttributeValue(XMLString value, 
                                       XMLString nonNormalizedValue,
                                       String atName,
                                       boolean checkEntities,String eleName)
@@ -810,9 +773,20 @@ public abstract class XMLScanner
             System.out.println("** scanLiteral -> \""
                                + value.toString() + "\"");
         }
+        
+        int fromIndex = 0;
+        if (c == quote && (fromIndex = isUnchangedByNormalization(value)) == -1) {
+            /** Both the non-normalized and normalized attribute values are equal. **/
+            nonNormalizedValue.setValues(value);
+            int cquote = fEntityScanner.scanChar();
+            if (cquote != quote) {
+                reportFatalError("CloseQuoteExpected", new Object[]{eleName,atName});
+            }
+            return true;
+        }
         fStringBuffer2.clear();
         fStringBuffer2.append(value);
-        normalizeWhitespace(value);
+        normalizeWhitespace(value, fromIndex);
         if (DEBUG_ATTR_NORMALIZATION) {
             System.out.println("** normalizeWhitespace -> \""
                                + value.toString() + "\"");
@@ -994,6 +968,8 @@ public abstract class XMLScanner
         if (cquote != quote) {
 			reportFatalError("CloseQuoteExpected", new Object[]{eleName,atName});
         }
+        return nonNormalizedValue.equals(value.ch, value.offset, value.length);
+        
     } // scanAttributeValue()
 
 
@@ -1139,7 +1115,7 @@ public abstract class XMLScanner
      */
     protected void normalizeWhitespace(XMLString value) {
         int end = value.offset + value.length;
-        for (int i = value.offset; i < end; i++) {
+        for (int i = value.offset; i < end; ++i) {
             int c = value.ch[i];
             // Performance: For XML 1.0 documents take advantage of 
             // the fact that the only legal characters below 0x20 
@@ -1152,6 +1128,50 @@ public abstract class XMLScanner
             }
         }
     }
+    
+    /**
+     * Normalize whitespace in an XMLString converting all whitespace
+     * characters to space characters.
+     */
+    protected void normalizeWhitespace(XMLString value, int fromIndex) {
+        int end = value.offset + value.length;
+        for (int i = value.offset + fromIndex; i < end; ++i) {
+            int c = value.ch[i];
+            // Performance: For XML 1.0 documents take advantage of 
+            // the fact that the only legal characters below 0x20 
+            // are 0x09 (TAB), 0x0A (LF) and 0x0D (CR). Since we've 
+            // already determined the well-formedness of these
+            // characters it is sufficient (and safe) to check
+            // against 0x20. -- mrglavas
+            if (c < 0x20) {
+                value.ch[i] = ' ';
+            }
+        }
+    }
+    
+    /**
+     * Checks whether this string would be unchanged by normalization.
+     * 
+     * @return -1 if the value would be unchanged by normalization,
+     * otherwise the index of the first whitespace character which
+     * would be transformed.
+     */
+    protected int isUnchangedByNormalization(XMLString value) {
+        int end = value.offset + value.length;
+        for (int i = value.offset; i < end; ++i) {
+            int c = value.ch[i];
+            // Performance: For XML 1.0 documents take advantage of 
+            // the fact that the only legal characters below 0x20 
+            // are 0x09 (TAB), 0x0A (LF) and 0x0D (CR). Since we've 
+            // already determined the well-formedness of these
+            // characters it is sufficient (and safe) to check
+            // against 0x20. -- mrglavas
+            if (c < 0x20) {
+                return i - value.offset;
+            }
+        }
+        return -1;
+    }
 
     //
     // XMLEntityHandler methods
@@ -1170,12 +1190,13 @@ public abstract class XMLScanner
      *                 where the entity encoding is not auto-detected (e.g.
      *                 internal entities or a document entity that is
      *                 parsed from a java.io.Reader).
+     * @param augs     Additional information that may include infoset augmentations
      *
      * @throws XNIException Thrown by handler to signal an error.
      */
     public void startEntity(String name, 
                             XMLResourceIdentifier identifier,
-                            String encoding) throws XNIException {
+                            String encoding, Augmentations augs) throws XNIException {
 
         // keep track of the entity depth
         fEntityDepth++;
@@ -1191,10 +1212,11 @@ public abstract class XMLScanner
      * specified by their name.
      * 
      * @param name The name of the entity.
+     * @param augs Additional information that may include infoset augmentations
      *
      * @throws XNIException Thrown by handler to signal an error.
      */
-    public void endEntity(String name) throws XNIException {
+    public void endEntity(String name, Augmentations augs) throws XNIException {
 
         // keep track of the entity depth
         fEntityDepth--;
