@@ -1,58 +1,17 @@
 /*
- * The Apache Software License, Version 1.1
- *
- *
- * Copyright (c) 2000-2004 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.apache.org.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2000-2004,2006 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package org.apache.xerces.impl.io;
@@ -60,16 +19,19 @@ package org.apache.xerces.impl.io;
 import java.io.InputStream;
 import java.io.IOException;
 import java.io.Reader;
-import java.io.UTFDataFormatException;
 
 import java.util.Locale;
 import org.apache.xerces.util.MessageFormatter;
 import org.apache.xerces.impl.msg.XMLMessageFormatter;
 
 /**
+ * <p>A UTF-8 reader.</p>
+ * 
+ * @xerces.internal
+ * 
  * @author Andy Clark, IBM
  *
- * @version $Id: UTF8Reader.java,v 1.8 2004/01/22 20:41:21 mrglavas Exp $
+ * @version $Id: UTF8Reader.java 422572 2006-07-16 23:48:08Z mrglavas $
  */
 public class UTF8Reader
     extends Reader {
@@ -91,10 +53,10 @@ public class UTF8Reader
     //
 
     /** Input stream. */
-    protected InputStream fInputStream;
+    protected final InputStream fInputStream;
 
     /** Byte buffer. */
-    protected byte[] fBuffer;
+    protected final byte[] fBuffer;
 
     /** Offset into buffer. */
     protected int fOffset;
@@ -104,10 +66,10 @@ public class UTF8Reader
 
     // message formatter; used to produce localized
     // exception messages
-    private MessageFormatter fFormatter = null;
+    private final MessageFormatter fFormatter;
 
     //Locale to use for messages
-    private Locale fLocale = null;
+    private final Locale fLocale;
 
     //
     // Constructors
@@ -134,7 +96,7 @@ public class UTF8Reader
     public UTF8Reader(InputStream inputStream, MessageFormatter messageFormatter,
             Locale locale) {
         this(inputStream, DEFAULT_BUFFER_SIZE, messageFormatter, locale);
-    } // <init>(InputStream, MessageFormatter)
+    } // <init>(InputStream, MessageFormatter, Locale)
 
     /**
      * Constructs a UTF-8 reader from the specified input stream,
@@ -147,11 +109,25 @@ public class UTF8Reader
      */
     public UTF8Reader(InputStream inputStream, int size,
             MessageFormatter messageFormatter, Locale locale) {
+        this(inputStream, new byte[size], messageFormatter, locale);
+    } // <init>(InputStream, int, MessageFormatter, Locale)
+    
+    /**
+     * Constructs a UTF-8 reader from the specified input stream,
+     * buffer and MessageFormatter.
+     *
+     * @param inputStream The input stream.
+     * @param buffer      The byte buffer.
+     * @param messageFormatter  the formatter for localizing/formatting errors.
+     * @param locale    the Locale to use for messages
+     */
+    public UTF8Reader(InputStream inputStream, byte [] buffer,
+            MessageFormatter messageFormatter, Locale locale) {
         fInputStream = inputStream;
-        fBuffer = new byte[size];
+        fBuffer = buffer;
         fFormatter = messageFormatter;
         fLocale = locale;
-    } // <init>(InputStream,int, MessageFormatter)
+    } // <init>(InputStream, byte[], MessageFormatter, Locale)
 
     //
     // Reader methods
@@ -695,34 +671,37 @@ public class UTF8Reader
 
     /** Throws an exception for expected byte. */
     private void expectedByte(int position, int count)
-        throws UTFDataFormatException {
+        throws MalformedByteSequenceException {
 
-        String message = fFormatter.formatMessage(fLocale, "ExpectedByte",
-                new Object[] {Integer.toString(position), Integer.toString(count)});
-        throw new UTFDataFormatException(message);
+        throw new MalformedByteSequenceException(fFormatter,
+            fLocale,
+            XMLMessageFormatter.XML_DOMAIN,
+            "ExpectedByte",
+            new Object[] {Integer.toString(position), Integer.toString(count)});
 
-    } // expectedByte(int,int,int)
+    } // expectedByte(int,int)
 
     /** Throws an exception for invalid byte. */
     private void invalidByte(int position, int count, int c)
-        throws UTFDataFormatException {
+        throws MalformedByteSequenceException {
 
-        String message = fFormatter.formatMessage(fLocale, "InvalidByte",
-                new Object [] {Integer.toString(position), Integer.toString(count)});
-        throw new UTFDataFormatException(message);
+        throw new MalformedByteSequenceException(fFormatter,
+            fLocale,
+            XMLMessageFormatter.XML_DOMAIN,
+            "InvalidByte", 
+            new Object [] {Integer.toString(position), Integer.toString(count)});
 
-    } // invalidByte(int,int,int,int)
+    } // invalidByte(int,int,int)
 
     /** Throws an exception for invalid surrogate bits. */
-    private void invalidSurrogate(int uuuuu) throws UTFDataFormatException {
+    private void invalidSurrogate(int uuuuu) throws MalformedByteSequenceException {
 
-        StringBuffer str = new StringBuffer();
-        str.append("high surrogate bits in UTF-8 sequence must not exceed 0x10 but found 0x");
-
-        String message = fFormatter.formatMessage(fLocale, "InvalidHighSurrogate",
-                new Object[] {Integer.toHexString(uuuuu)});
-        throw new UTFDataFormatException(message);
+        throw new MalformedByteSequenceException(fFormatter,
+            fLocale,
+            XMLMessageFormatter.XML_DOMAIN,
+            "InvalidHighSurrogate", 
+            new Object[] {Integer.toHexString(uuuuu)});
 
     } // invalidSurrogate(int)
 
-} // class UTF8Reader
\ No newline at end of file
+} // class UTF8Reader
