@@ -1,58 +1,17 @@
 /*
- * The Apache Software License, Version 1.1
- *
- *
- * Copyright (c) 2001-2004 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.apache.org.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 2001-2006 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package org.apache.xerces.impl.xs.traversers;
@@ -68,8 +27,10 @@ import org.apache.xerces.impl.xs.XSElementDecl;
 import org.apache.xerces.impl.xs.XSParticleDecl;
 import org.apache.xerces.xs.XSConstants;
 import org.apache.xerces.xs.XSObject;
+import org.apache.xerces.xs.XSObjectList;
 import org.apache.xerces.xs.XSTypeDefinition;
 import org.apache.xerces.impl.xs.util.XInt;
+import org.apache.xerces.impl.xs.util.XSObjectListImpl;
 import org.apache.xerces.util.DOMUtil;
 import org.apache.xerces.util.SymbolTable;
 import org.apache.xerces.xni.QName;
@@ -97,23 +58,25 @@ import org.w3c.dom.Attr;
  *   Content: (annotation?, ((simpleType | complexType)?, (unique | key | keyref)*))
  * </element>
  *
+ * @xerces.internal 
+ *
  * @author Sandy Gao, IBM
  *
- * @version $Id: XSDElementTraverser.java,v 1.29 2004/02/03 17:27:45 sandygao Exp $
+ * @version $Id: XSDElementTraverser.java 442099 2006-09-11 05:01:27Z mrglavas $
  */
 class XSDElementTraverser extends XSDAbstractTraverser {
-
+    
     protected final XSElementDecl  fTempElementDecl  = new XSElementDecl();
-
+    
     // this controls what happens when a local element is encountered.
     // We may not encounter all local elements when first parsing.
     boolean fDeferTraversingLocalElements;
-
+    
     XSDElementTraverser (XSDHandler handler,
-                         XSAttributeChecker gAttrCheck) {
+            XSAttributeChecker gAttrCheck) {
         super(handler, gAttrCheck);
     }
-
+    
     /**
      * Traverse a locally declared element (or an element reference).
      *
@@ -128,11 +91,11 @@ class XSDElementTraverser extends XSDAbstractTraverser {
      * @return the particle
      */
     XSParticleDecl traverseLocal(Element elmDecl,
-                                 XSDocumentInfo schemaDoc,
-                                 SchemaGrammar grammar,
-                                 int allContextFlags,
-                                 XSObject parent) {
-
+            XSDocumentInfo schemaDoc,
+            SchemaGrammar grammar,
+            int allContextFlags,
+            XSObject parent) {
+        
         XSParticleDecl particle = null;
         if (fSchemaHandler.fDeclPool !=null) {
             particle = fSchemaHandler.fDeclPool.getParticleDecl();
@@ -157,15 +120,15 @@ class XSDElementTraverser extends XSDAbstractTraverser {
             }
             fSchemaHandler.fillInLocalElemInfo(elmDecl, schemaDoc, allContextFlags, parent, particle);
         } else {
-            traverseLocal(particle, elmDecl, schemaDoc, grammar, allContextFlags, parent);
+            traverseLocal(particle, elmDecl, schemaDoc, grammar, allContextFlags, parent, null);
             // If it's an empty particle, return null.
             if (particle.fType == XSParticleDecl.PARTICLE_EMPTY)
                 particle = null;
         }
-
+        
         return particle;
     }
-
+    
     /**
      * Traverse a locally declared element (or an element reference).
      *
@@ -175,31 +138,41 @@ class XSDElementTraverser extends XSDAbstractTraverser {
      * @param  index
      */
     protected void traverseLocal(XSParticleDecl particle,
-                                 Element elmDecl,
-                                 XSDocumentInfo schemaDoc,
-                                 SchemaGrammar grammar,
-                                 int allContextFlags,
-                                 XSObject parent) {
-
+            Element elmDecl,
+            XSDocumentInfo schemaDoc,
+            SchemaGrammar grammar,
+            int allContextFlags,
+            XSObject parent,
+            String[] localNSDecls) {
+        
+        if (localNSDecls != null) {
+            schemaDoc.fNamespaceSupport.setEffectiveContext(localNSDecls);
+        }
+        
         // General Attribute Checking
         Object[] attrValues = fAttrChecker.checkAttributes(elmDecl, false, schemaDoc);
-
+        
         QName refAtt = (QName) attrValues[XSAttributeChecker.ATTIDX_REF];
         XInt  minAtt = (XInt)  attrValues[XSAttributeChecker.ATTIDX_MINOCCURS];
         XInt  maxAtt = (XInt)  attrValues[XSAttributeChecker.ATTIDX_MAXOCCURS];
-
+        
         XSElementDecl element = null;
+        XSAnnotationImpl annotation = null;
         if (elmDecl.getAttributeNode(SchemaSymbols.ATT_REF) != null) {
             if (refAtt != null) {
                 element = (XSElementDecl)fSchemaHandler.getGlobalDecl(schemaDoc, XSDHandler.ELEMENT_TYPE, refAtt, elmDecl);
-
+                
                 Element child = DOMUtil.getFirstChildElement(elmDecl);
                 if (child != null && DOMUtil.getLocalName(child).equals(SchemaSymbols.ELT_ANNOTATION)) {
-                    // REVISIT:  put this somewhere
-                    traverseAnnotationDecl(child, attrValues, false, schemaDoc);
+                    annotation = traverseAnnotationDecl(child, attrValues, false, schemaDoc);
                     child = DOMUtil.getNextSiblingElement(child);
+                } 
+                else {
+                    String text = DOMUtil.getSyntheticAnnotation(elmDecl);
+                    if (text != null) {
+                        annotation = traverseSyntheticAnnotation(elmDecl, text, attrValues, false, schemaDoc);
+                    }
                 }
-
                 // Element Declaration Representation OK
                 // 2 If the item's parent is not <schema>, then all of the following must be true:
                 // 2.1 One of ref or name must be present, but not both.
@@ -213,7 +186,7 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         } else {
             element = traverseNamedElement(elmDecl, attrValues, schemaDoc, grammar, false, parent);
         }
-
+        
         particle.fMinOccurs = minAtt.intValue();
         particle.fMaxOccurs = maxAtt.intValue();
         if (element != null) {
@@ -223,14 +196,27 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         else {
             particle.fType = XSParticleDecl.PARTICLE_EMPTY;
         }
+        if (refAtt != null) {
+            XSObjectList annotations;
+            if (annotation != null) {
+                annotations = new XSObjectListImpl();
+                ((XSObjectListImpl) annotations).add(annotation);
+            } else {
+                annotations = XSObjectListImpl.EMPTY_LIST;
+            }
+            particle.fAnnotations = annotations;
+        } else {
+            particle.fAnnotations = ((element != null) ? element.fAnnotations
+                    : XSObjectListImpl.EMPTY_LIST);
+        }
         Long defaultVals = (Long)attrValues[XSAttributeChecker.ATTIDX_FROMDEFAULT];
         checkOccurrences(particle, SchemaSymbols.ELT_ELEMENT,
-                         (Element)elmDecl.getParentNode(), allContextFlags,
-                         defaultVals.longValue());
-
+                (Element)elmDecl.getParentNode(), allContextFlags,
+                defaultVals.longValue());
+        
         fAttrChecker.returnAttrArray(attrValues, schemaDoc);
     }
-
+    
     /**
      * Traverse a globally declared element.
      *
@@ -240,17 +226,18 @@ class XSDElementTraverser extends XSDAbstractTraverser {
      * @return the element declaration
      */
     XSElementDecl traverseGlobal(Element elmDecl,
-                                 XSDocumentInfo schemaDoc,
-                                 SchemaGrammar grammar) {
+            XSDocumentInfo schemaDoc,
+            SchemaGrammar grammar) {
+        
+        // General Attribute Checking'
 
-        // General Attribute Checking
         Object[] attrValues = fAttrChecker.checkAttributes(elmDecl, true, schemaDoc);
         XSElementDecl element = traverseNamedElement(elmDecl, attrValues, schemaDoc, grammar, true, null);
         fAttrChecker.returnAttrArray(attrValues, schemaDoc);
         return element;
-
+        
     }
-
+    
     /**
      * Traverse a globally declared element.
      *
@@ -262,12 +249,12 @@ class XSDElementTraverser extends XSDAbstractTraverser {
      * @return the element declaration
      */
     XSElementDecl traverseNamedElement(Element elmDecl,
-                                       Object[] attrValues,
-                                       XSDocumentInfo schemaDoc,
-                                       SchemaGrammar grammar,
-                                       boolean isGlobal,
-                                       XSObject parent) {
-
+            Object[] attrValues,
+            XSDocumentInfo schemaDoc,
+            SchemaGrammar grammar,
+            boolean isGlobal,
+            XSObject parent) {
+        
         Boolean abstractAtt  = (Boolean) attrValues[XSAttributeChecker.ATTIDX_ABSTRACT];
         XInt    blockAtt     = (XInt)    attrValues[XSAttributeChecker.ATTIDX_BLOCK];
         String  defaultAtt   = (String)  attrValues[XSAttributeChecker.ATTIDX_DEFAULT];
@@ -278,9 +265,9 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         Boolean nillableAtt  = (Boolean) attrValues[XSAttributeChecker.ATTIDX_NILLABLE];
         QName   subGroupAtt  = (QName)   attrValues[XSAttributeChecker.ATTIDX_SUBSGROUP];
         QName   typeAtt      = (QName)   attrValues[XSAttributeChecker.ATTIDX_TYPE];
-
+        
         // Step 1: get declaration information
-
+        
         XSElementDecl element = null;
         if (fSchemaHandler.fDeclPool !=null) {
             element = fSchemaHandler.fDeclPool.getElementDecl();
@@ -290,7 +277,7 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         // get 'name'
         if (nameAtt != null)
             element.fName = fSymbolTable.addSymbol(nameAtt);
-
+        
         // get 'target namespace'
         if (isGlobal) {
             element.fTargetNamespace = schemaDoc.fTargetNamespace;
@@ -299,7 +286,7 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         else {
             if (parent instanceof XSComplexTypeDecl)
                 element.setIsLocal((XSComplexTypeDecl)parent);
-
+            
             if (formAtt != null) {
                 if (formAtt.intValue() == SchemaSymbols.FORM_QUALIFIED)
                     element.fTargetNamespace = schemaDoc.fTargetNamespace;
@@ -311,19 +298,19 @@ class XSDElementTraverser extends XSDAbstractTraverser {
                 element.fTargetNamespace = null;
             }
         }
-
+        
         // get 'block', 'final', 'nillable', 'abstract'
         element.fBlock = blockAtt == null ? schemaDoc.fBlockDefault : blockAtt.shortValue();
         element.fFinal = finalAtt == null ? schemaDoc.fFinalDefault : finalAtt.shortValue();
         // discard valid Block/Final 'Default' values that are invalid for Block/Final
         element.fBlock &= (XSConstants.DERIVATION_EXTENSION | XSConstants.DERIVATION_RESTRICTION | XSConstants.DERIVATION_SUBSTITUTION);
         element.fFinal &= (XSConstants.DERIVATION_EXTENSION | XSConstants.DERIVATION_RESTRICTION);
-
+        
         if (nillableAtt.booleanValue())
             element.setIsNillable();
         if (abstractAtt != null && abstractAtt.booleanValue())
             element.setIsAbstract();
-
+        
         // get 'value constraint'
         if (fixedAtt != null) {
             element.fDefault = new ValidatedInfo();
@@ -336,12 +323,12 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         } else {
             element.setConstraintType(XSConstants.VC_NONE);
         }
-
+        
         // get 'substitutionGroup affiliation'
         if (subGroupAtt != null) {
             element.fSubGroup = (XSElementDecl)fSchemaHandler.getGlobalDecl(schemaDoc, XSDHandler.ELEMENT_TYPE, subGroupAtt, elmDecl);
         }
-
+        
         // get 'annotation'
         Element child = DOMUtil.getFirstChildElement(elmDecl);
         XSAnnotationImpl annotation = null;
@@ -349,16 +336,30 @@ class XSDElementTraverser extends XSDAbstractTraverser {
             annotation = traverseAnnotationDecl(child, attrValues, false, schemaDoc);
             child = DOMUtil.getNextSiblingElement(child);
         }
-        element.fAnnotation = annotation;
-
+        else {
+            String text = DOMUtil.getSyntheticAnnotation(elmDecl);
+            if (text != null) {
+                annotation = traverseSyntheticAnnotation(elmDecl, text, attrValues, false, schemaDoc);
+            }
+        }
+        
+        XSObjectList annotations;
+        if (annotation != null) {
+            annotations = new XSObjectListImpl();
+            ((XSObjectListImpl)annotations).add (annotation);
+        } else {
+            annotations = XSObjectListImpl.EMPTY_LIST;
+        }
+        element.fAnnotations = annotations;
+        
         // get 'type definition'
         XSTypeDefinition elementType = null;
         boolean haveAnonType = false;
-
+        
         // Handle Anonymous type if there is one
         if (child != null) {
             String childName = DOMUtil.getLocalName(child);
-
+            
             if (childName.equals(SchemaSymbols.ELT_COMPLEXTYPE)) {
                 elementType = fSchemaHandler.fComplexTypeTraverser.traverseLocal(child, schemaDoc, grammar);
                 haveAnonType = true;
@@ -370,33 +371,33 @@ class XSDElementTraverser extends XSDAbstractTraverser {
                 child = DOMUtil.getNextSiblingElement(child);
             }
         }
-
+        
         // Handler type attribute
         if (elementType == null && typeAtt != null) {
             elementType = (XSTypeDefinition)fSchemaHandler.getGlobalDecl(schemaDoc, XSDHandler.TYPEDECL_TYPE, typeAtt, elmDecl);
         }
-
+        
         // Get it from the substitutionGroup declaration
         if (elementType == null && element.fSubGroup != null) {
             elementType = element.fSubGroup.fType;
         }
-
+        
         if (elementType == null) {
             elementType = SchemaGrammar.fAnyType;
         }
-
+        
         element.fType = elementType;
-
+        
         // get 'identity constraint'
-
+        
         // see if there's something here; it had better be key, keyref or unique.
         if (child != null) {
             String childName = DOMUtil.getLocalName(child);
             while (child != null &&
-                   (childName.equals(SchemaSymbols.ELT_KEY) ||
-                    childName.equals(SchemaSymbols.ELT_KEYREF) ||
-                    childName.equals(SchemaSymbols.ELT_UNIQUE))) {
-
+                    (childName.equals(SchemaSymbols.ELT_KEY) ||
+                            childName.equals(SchemaSymbols.ELT_KEYREF) ||
+                            childName.equals(SchemaSymbols.ELT_UNIQUE))) {
+                
                 if (childName.equals(SchemaSymbols.ELT_KEY) ||
                         childName.equals(SchemaSymbols.ELT_UNIQUE)) {
                     // need to set <key>/<unique> to hidden before traversing it,
@@ -405,10 +406,10 @@ class XSDElementTraverser extends XSDAbstractTraverser {
                     fSchemaHandler.fUniqueOrKeyTraverser.traverse(child, element, schemaDoc, grammar);
                     if(DOMUtil.getAttrValue(child, SchemaSymbols.ATT_NAME).length() != 0 ) {
                         fSchemaHandler.checkForDuplicateNames(
-                            (schemaDoc.fTargetNamespace == null) ? ","+DOMUtil.getAttrValue(child, SchemaSymbols.ATT_NAME)
-                            : schemaDoc.fTargetNamespace+","+ DOMUtil.getAttrValue(child, SchemaSymbols.ATT_NAME),
-                            fSchemaHandler.getIDRegistry(),
-                            child, schemaDoc);
+                                (schemaDoc.fTargetNamespace == null) ? ","+DOMUtil.getAttrValue(child, SchemaSymbols.ATT_NAME)
+                                        : schemaDoc.fTargetNamespace+","+ DOMUtil.getAttrValue(child, SchemaSymbols.ATT_NAME),
+                                        fSchemaHandler.getIDRegistry(), fSchemaHandler.getIDRegistry_sub(),
+                                        child, schemaDoc);
                     }
                 } else if (childName.equals(SchemaSymbols.ELT_KEYREF)) {
                     fSchemaHandler.storeKeyRef(child, schemaDoc, element);
@@ -419,13 +420,13 @@ class XSDElementTraverser extends XSDAbstractTraverser {
                 }
             }
         }
-
+        
         // Step 2: register the element decl to the grammar
         if (isGlobal && nameAtt != null)
             grammar.addGlobalElementDecl(element);
-
+        
         // Step 3: check against schema for schemas
-
+        
         // required attributes
         if (nameAtt == null) {
             if (isGlobal)
@@ -434,39 +435,39 @@ class XSDElementTraverser extends XSDAbstractTraverser {
                 reportSchemaError("src-element.2.1", null, elmDecl);
             nameAtt = NO_NAME;
         }
-
+        
         // element
         if (child != null) {
             reportSchemaError("s4s-elt-must-match.1", new Object[]{nameAtt, "(annotation?, (simpleType | complexType)?, (unique | key | keyref)*))", DOMUtil.getLocalName(child)}, child);
         }
-
+        
         // Step 4: check 3.3.3 constraints
-
+        
         // src-element
-
+        
         // 1 default and fixed must not both be present.
         if (defaultAtt != null && fixedAtt != null) {
             reportSchemaError("src-element.1", new Object[]{nameAtt}, elmDecl);
         }
-
+        
         // 2 If the item's parent is not <schema>, then all of the following must be true:
         // 2.1 One of ref or name must be present, but not both.
         // This is checked in XSAttributeChecker
-
+        
         // 2.2 If ref is present, then all of <complexType>, <simpleType>, <key>, <keyref>, <unique>, nillable, default, fixed, form, block and type must be absent, i.e. only minOccurs, maxOccurs, id are allowed in addition to ref, along with <annotation>.
         // Attributes are checked in XSAttributeChecker, elements are checked in "traverse" method
-
+        
         // 3 type and either <simpleType> or <complexType> are mutually exclusive.
         if (haveAnonType && (typeAtt != null)) {
             reportSchemaError("src-element.3", new Object[]{nameAtt}, elmDecl);
         }
-
+        
         // Step 5: check 3.3.6 constraints
         // check for NOTATION type
         checkNotationType(nameAtt, elementType, elmDecl);
-
+        
         // e-props-correct
-
+        
         // 2 If there is a {value constraint}, the canonical lexical representation of its value must be valid with respect to the {type definition} as defined in Element Default Valid (Immediate) (3.3.6).
         if (element.fDefault != null) {
             fValidationState.setNamespaceSupport(schemaDoc.fNamespaceSupport);
@@ -475,34 +476,34 @@ class XSDElementTraverser extends XSDAbstractTraverser {
                 element.setConstraintType(XSConstants.VC_NONE);
             }
         }
-
+        
         // 4 If there is an {substitution group affiliation}, the {type definition} of the element declaration must be validly derived from the {type definition} of the {substitution group affiliation}, given the value of the {substitution group exclusions} of the {substitution group affiliation}, as defined in Type Derivation OK (Complex) (3.4.6) (if the {type definition} is complex) or as defined in Type Derivation OK (Simple) (3.14.6) (if the {type definition} is simple).
         if (element.fSubGroup != null) {
-           if (!XSConstraints.checkTypeDerivationOk(element.fType, element.fSubGroup.fType, element.fSubGroup.fFinal)) {
+            if (!XSConstraints.checkTypeDerivationOk(element.fType, element.fSubGroup.fType, element.fSubGroup.fFinal)) {
                 reportSchemaError ("e-props-correct.4", new Object[]{nameAtt, subGroupAtt.prefix+":"+subGroupAtt.localpart}, elmDecl);
-           }
+            }
         }
-
+        
         // 5 If the {type definition} or {type definition}'s {content type} is or is derived from ID then there must not be a {value constraint}.
         if (element.fDefault != null) {
             if ((elementType.getTypeCategory() == XSTypeDefinition.SIMPLE_TYPE &&
-                 ((XSSimpleType)elementType).isIDType()) ||
-                (elementType.getTypeCategory() == XSTypeDefinition.COMPLEX_TYPE &&
-                 ((XSComplexTypeDecl)elementType).containsTypeID())) {
+                    ((XSSimpleType)elementType).isIDType()) ||
+                    (elementType.getTypeCategory() == XSTypeDefinition.COMPLEX_TYPE &&
+                            ((XSComplexTypeDecl)elementType).containsTypeID())) {
                 reportSchemaError ("e-props-correct.5", new Object[]{element.fName}, elmDecl);
             }
         }
-
+        
         // Element without a name. Return null.
         if (element.fName == null)
             return null;
-
+        
         return element;
     }
-
-    void reset(SymbolTable symbolTable) {
-        super.reset(symbolTable);
+    
+    void reset(SymbolTable symbolTable, boolean validateAnnotations) {
+        super.reset(symbolTable, validateAnnotations);
         fDeferTraversingLocalElements = true;
     } // reset()
-
+    
 }
