@@ -1,61 +1,19 @@
 /*
- * The Apache Software License, Version 1.1
- *
- *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.apache.org.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Copyright 1999-2002,2004-2006 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
-
 // Sep 14, 2000:
 //  Fixed comments to preserve whitespaces and add a line break
 //  when indenting. Reported by Gervase Markham <gerv@gerv.net>
@@ -80,10 +38,8 @@
 //   entering element state.
 //   Reported by Lowell Vaughn <lvaughn@agillion.com>
 
-
 package org.apache.xml.serialize;
 
-
 import java.io.IOException;
 import java.io.OutputStream;
 import java.io.Writer;
@@ -93,18 +49,19 @@ import java.util.Vector;
 import org.apache.xerces.dom.DOMErrorImpl;
 import org.apache.xerces.dom.DOMLocatorImpl;
 import org.apache.xerces.dom.DOMMessageFormatter;
-import org.apache.xerces.dom3.DOMError;
-import org.apache.xerces.dom3.DOMErrorHandler;
 import org.apache.xerces.util.XMLChar;
 import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Document;
 import org.w3c.dom.DocumentFragment;
 import org.w3c.dom.DocumentType;
+import org.w3c.dom.DOMError;
+import org.w3c.dom.DOMErrorHandler;
 import org.w3c.dom.Element;
 import org.w3c.dom.Entity;
 import org.w3c.dom.NamedNodeMap;
 import org.w3c.dom.Node;
 import org.w3c.dom.Notation;
+import org.w3c.dom.ls.LSException;
 import org.w3c.dom.ls.LSSerializerFilter;
 import org.w3c.dom.traversal.NodeFilter;
 import org.xml.sax.ContentHandler;
@@ -157,7 +114,7 @@ import org.xml.sax.ext.LexicalHandler;
  * another element.
  *
  *
- * @version $Revision: 1.51 $ $Date: 2004/02/12 16:56:07 $
+ * @version $Revision: 410017 $ $Date: 2006-05-28 23:53:02 -0400 (Sun, 28 May 2006) $
  * @author <a href="mailto:arkin@intalio.com">Assaf Arkin</a>
  * @author <a href="mailto:rahul.srivastava@sun.com">Rahul Srivastava</a>
  * @author Elena Litani, IBM 
@@ -536,9 +493,10 @@ public abstract class BaseMarkupSerializer
             saveIndent = _printer.getNextIndent();
             _printer.setNextIndent( 0 );
             char ch;
-            for ( int index = start ; index < length ; ++index ) {
+            final int end = start + length;
+            for ( int index = start ; index < end; ++index ) {
                 ch = chars[index];
-                if ( ch == ']' && index + 2 < length &&
+                if ( ch == ']' && index + 2 < end &&
                      chars[ index + 1 ] == ']' && chars[ index + 2 ] == '>' ) {
                     _printer.printText("]]]]><![CDATA[>");
                     index +=2; 
@@ -546,8 +504,8 @@ public abstract class BaseMarkupSerializer
                 }
                 if (!XMLChar.isValid(ch)) {
                     // check if it is surrogate
-                    if (++index <length) {
-                        surrogates(ch, chars[index]);
+                    if (++index < end) {
+                        surrogates(ch, chars[index], true);
                     } 
                     else {
                         fatalError("The character '"+(char)ch+"' is an invalid XML character"); 
@@ -1127,7 +1085,8 @@ public abstract class BaseMarkupSerializer
             endCDATA();
             content();
             
-            if ( ((features & DOMSerializerImpl.ENTITIES) != 0)){
+            if (((features & DOMSerializerImpl.ENTITIES) != 0)
+                || (node.getFirstChild() == null)) {
                 if (fDOMFilter !=null && 
                       (fDOMFilter.getWhatToShow() & NodeFilter.SHOW_ENTITY_REFERENCE)!= 0) {
                       short code = fDOMFilter.acceptNode(node);
@@ -1240,18 +1199,18 @@ public abstract class BaseMarkupSerializer
                     String docTypePublicId = null;
                     String docTypeSystemId = null;
                     try {
-                        java.lang.reflect.Method getPublicId = docTypeClass.getMethod("getPublicId", null);
+                        java.lang.reflect.Method getPublicId = docTypeClass.getMethod("getPublicId", (Class[]) null);
                         if (getPublicId.getReturnType().equals(String.class)) {
-                            docTypePublicId = (String)getPublicId.invoke(docType, null);
+                            docTypePublicId = (String)getPublicId.invoke(docType, (Object[]) null);
                         }
                     }
                     catch (Exception e) {
                         // ignore
                     }
                     try {
-                        java.lang.reflect.Method getSystemId = docTypeClass.getMethod("getSystemId", null);
+                        java.lang.reflect.Method getSystemId = docTypeClass.getMethod("getSystemId", (Class[]) null);
                         if (getSystemId.getReturnType().equals(String.class)) {
-                            docTypeSystemId = (String)getSystemId.invoke(docType, null);
+                            docTypeSystemId = (String)getSystemId.invoke(docType, (Object[]) null);
                         }
                     }
                     catch (Exception e) {
@@ -1452,23 +1411,24 @@ public abstract class BaseMarkupSerializer
                 if (fDOMErrorHandler != null) {
                     // REVISIT: this means that if DOM Error handler is not registered we don't report any
                     // fatal errors and might serialize not wellformed document
-                    if ((features & DOMSerializerImpl.SPLITCDATA) == 0
-                        && (features & DOMSerializerImpl.WELLFORMED) == 0) {
-                        // issue fatal error
-                        String msg =
-                            DOMMessageFormatter.formatMessage(
-                                DOMMessageFormatter.SERIALIZER_DOMAIN,
-                                "EndingCDATA",
-                                null);
-                        modifyDOMError(
-                            msg,
-                            DOMError.SEVERITY_FATAL_ERROR,
-                            fCurrentNode);
-                        boolean continueProcess =
+                    if ((features & DOMSerializerImpl.SPLITCDATA) == 0) {
+                        String msg = DOMMessageFormatter.formatMessage(
+                            DOMMessageFormatter.SERIALIZER_DOMAIN,
+                            "EndingCDATA",
+                            null);    
+                        if ((features & DOMSerializerImpl.WELLFORMED) != 0) {
+                            // issue fatal error
+                            modifyDOMError(msg, DOMError.SEVERITY_FATAL_ERROR, "wf-invalid-character", fCurrentNode);
                             fDOMErrorHandler.handleError(fDOMError);
-                        if (!continueProcess) {
-                            throw new IOException();
-                        }
+                            throw new LSException(LSException.SERIALIZE_ERR, msg);
+                        } 
+                        else {
+                            // issue error
+                            modifyDOMError(msg, DOMError.SEVERITY_ERROR, "cdata-section-not-splitted", fCurrentNode);
+                            if (!fDOMErrorHandler.handleError(fDOMError)) {
+                                throw new LSException(LSException.SERIALIZE_ERR, msg);
+                            }
+                        }                        
                     } else {
                         // issue warning
                         String msg =
@@ -1479,7 +1439,7 @@ public abstract class BaseMarkupSerializer
                         modifyDOMError(
                             msg,
                             DOMError.SEVERITY_WARNING,
-                            fCurrentNode);
+                            null, fCurrentNode);
                         fDOMErrorHandler.handleError(fDOMError);
                     }
                 }
@@ -1492,7 +1452,7 @@ public abstract class BaseMarkupSerializer
             if (!XMLChar.isValid(ch)) {
                 // check if it is surrogate
                 if (++index <length) {
-                    surrogates(ch, text.charAt(index));
+                    surrogates(ch, text.charAt(index), true);
                 } 
                 else {
                     fatalError("The character '"+(char)ch+"' is an invalid XML character"); 
@@ -1514,7 +1474,7 @@ public abstract class BaseMarkupSerializer
     }
 
 
-    protected void surrogates(int high, int low) throws IOException{
+    protected void surrogates(int high, int low, boolean inContent) throws IOException{
         if (XMLChar.isHighSurrogate(high)) {
             if (!XMLChar.isLowSurrogate(low)) {
                 //Invalid XML
@@ -1527,7 +1487,7 @@ public abstract class BaseMarkupSerializer
                     fatalError("The character '"+(char)supplemental+"' is an invalid XML character"); 
                 }
                 else {
-                    if (content().inCData ) {
+                    if (inContent && content().inCData) {
                         _printer.printText("]]>&#x");                        
                         _printer.printText(Integer.toHexString(supplemental));                        
                         _printer.printText(";<![CDATA[");
@@ -1813,7 +1773,11 @@ public abstract class BaseMarkupSerializer
     {
         return _elementStateCount == 0;
     }
-
+    
+    /** Clears document state. **/
+    final void clearDocumentState() {
+        _elementStateCount = 0;
+    }
 
     /**
      * Returns the namespace prefix for the specified URI.
@@ -1851,11 +1815,13 @@ public abstract class BaseMarkupSerializer
      * 
      * @param message
      * @param severity
+     * @param type
      * @return a DOMError
      */
-    protected DOMError modifyDOMError(String message, short severity, Node node){
+    protected DOMError modifyDOMError(String message, short severity, String type, Node node){
             fDOMError.reset();
             fDOMError.fMessage = message;
+            fDOMError.fType = type;
             fDOMError.fSeverity = severity;
             fDOMError.fLocator = new DOMLocatorImpl(-1, -1, -1, node, null);
             return fDOMError;
@@ -1865,7 +1831,7 @@ public abstract class BaseMarkupSerializer
 
     protected void fatalError(String message) throws IOException{
         if (fDOMErrorHandler != null) {
-            modifyDOMError(message, DOMError.SEVERITY_FATAL_ERROR, fCurrentNode);
+            modifyDOMError(message, DOMError.SEVERITY_FATAL_ERROR, null, fCurrentNode);
             fDOMErrorHandler.handleError(fDOMError);
         } 
         else {
